"""Module for subtitle list panel

"""

# from multiprocessing.spawn import old_main_modules
import os

from PySide6.QtWidgets import QHBoxLayout, QLayout, QPushButton, QLabel, QFileDialog, QMessageBox, QSizePolicy, QStackedWidget, QVBoxLayout, QWidget, QLineEdit
from PySide6.QtCore import QPropertyAnimation, QEasingCurve, Qt, QSize
from PySide6.QtGui import QTextCursor

from subtitld.interface import subtitles_panel_widget_markdown, subtitles_panel_widget_qlistwidget, subtitles_panel_widget_timeline, timeline
from subtitld.modules import file_io
from subtitld.modules import subtitles
from subtitld.modules.paths import LIST_OF_SUPPORTED_SUBTITLE_EXTENSIONS


def load(self):
    """Function to load subtitles list widgets"""
    self.subtitles_panel_widget = QLabel(parent=self)
    self.subtitles_panel_widget.setObjectName('subtitles_panel_widget')
    self.subtitles_panel_widget_animation = QPropertyAnimation(self.subtitles_panel_widget, b'geometry')
    self.subtitles_panel_widget_animation.setEasingCurve(QEasingCurve.OutCirc)
    self.subtitles_panel_widget.setAttribute(Qt.WA_LayoutOnEntireRect)
    self.subtitles_panel_widget.setLayout(QHBoxLayout())
    self.subtitles_panel_widget.layout().setContentsMargins(0, 20, 2, 210)
    self.subtitles_panel_widget.layout().setSpacing(0)

    self.subtitles_panel_widget_vbox = QVBoxLayout()
    self.subtitles_panel_widget_vbox.setContentsMargins(0, 0, 0, 0)
    self.subtitles_panel_widget_vbox.setSpacing(20)

    self.subtitles_panel_widget_top_bar = QHBoxLayout()
    self.subtitles_panel_widget_top_bar.setSpacing(8)
    self.subtitles_panel_widget_top_bar.setContentsMargins(0, 0, 0, 0)

    self.toppanel_format_label = QLabel()
    self.toppanel_format_label.setObjectName('toppanel_format_label')
    self.toppanel_format_label.setProperty('class', 'unsaved')
    self.toppanel_format_label.setLayout(QHBoxLayout())
    # self.toppanel_format_label.setSizePolicy(QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Maximum))
    # self.toppanel_format_label.setMinimumHeight(40)
    self.toppanel_format_label.layout().setSpacing(8)
    self.toppanel_format_label.layout().setContentsMargins(20, 5, 5, 5)
    self.toppanel_format_label.layout().setSizeConstraint(QLayout.SetMinAndMaxSize)

    self.toppanel_format_label_text = QLabel()
    self.toppanel_format_label_text.setObjectName('toppanel_format_label_text')
    self.toppanel_format_label_text.setSizePolicy(QSizePolicy(QSizePolicy.Maximum, QSizePolicy.Minimum))
    self.toppanel_format_label.layout().addWidget(self.toppanel_format_label_text, 0)

    self.toppanel_save_button = QPushButton()
    self.toppanel_save_button.setObjectName('toppanel_save_button')
    self.toppanel_save_button.clicked.connect(lambda: toppanel_save_button_clicked(self))
    self.toppanel_save_button.setProperty('class', 'subbutton2_dark')
    # self.toppanel_save_button.setFixedSize(QSize(48, 48))
    self.toppanel_save_button.setSizePolicy(QSizePolicy(QSizePolicy.Maximum, QSizePolicy.Minimum))
    self.toppanel_format_label.layout().addWidget(self.toppanel_save_button, 0)

    self.subtitles_panel_widget_top_bar.addWidget(self.toppanel_format_label, 0)

    class toppanel_subtitle_file_info_label(QLabel):
        def enterEvent(widget, event):
            self.toppanel_open_button.setVisible(True)
            event.accept()

        def leaveEvent(widget, event):
            self.toppanel_open_button.setVisible(False)
            event.accept()

    self.toppanel_subtitle_file_info_label = toppanel_subtitle_file_info_label()
    self.toppanel_subtitle_file_info_label.setLayout(QHBoxLayout(self.toppanel_subtitle_file_info_label))
    self.toppanel_subtitle_file_info_label.setSizePolicy(QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Maximum))
    self.toppanel_subtitle_file_info_label.layout().setContentsMargins(0, 0, 0, 0)
    self.toppanel_subtitle_file_info_label.setObjectName('toppanel_subtitle_file_info_label')

    self.toppanel_open_button = QPushButton('Open different file'.upper())
    self.toppanel_open_button.setObjectName('toppanel_open_button')
    self.toppanel_open_button.setProperty('class', 'subbutton2_dark')
    self.toppanel_open_button.clicked.connect(lambda: toppanel_open_button_clicked(self))
    # self.toppanel_open_button.setSizePolicy(QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Maximum))
    # self.toppanel_open_button.setProperty('class', 'button')
    self.toppanel_open_button.setVisible(False)
    self.toppanel_subtitle_file_info_label.layout().addWidget(self.toppanel_open_button, 0, Qt.AlignRight)

    self.subtitles_panel_widget_top_bar.addWidget(self.toppanel_subtitle_file_info_label, 1)

    self.subtitles_panel_widget_vbox.layout().addLayout(self.subtitles_panel_widget_top_bar)

    self.subtitles_panel_stackedwidgets = QStackedWidget()

    subtitles_panel_widget_qlistwidget.add_widgets(self)

    subtitles_panel_widget_markdown.add_widgets(self)

    subtitles_panel_widget_timeline.add_widgets(self)

    self.subtitles_panel_widget_vbox.layout().addWidget(self.subtitles_panel_stackedwidgets)
    self.subtitles_panel_widget.layout().addLayout(self.subtitles_panel_widget_vbox)

    self.subtitles_panel_widget_buttons_vbox = QVBoxLayout()
    self.subtitles_panel_widget_buttons_vbox.setContentsMargins(10, 51, 0, 0)
    self.subtitles_panel_widget_buttons_vbox.setSpacing(0)

    subtitles_panel_widget_qlistwidget.add_button(self)

    self.subtitles_panel_widget_buttons_vbox.addSpacing(-10)

    subtitles_panel_widget_markdown.add_button(self)

    self.subtitles_panel_widget_buttons_vbox.addSpacing(-10)

    subtitles_panel_widget_timeline.add_button(self)

    self.subtitles_panel_widget_buttons_vbox.addStretch()

    self.subtitles_panel_findandreplace_list = []
    self.subtitles_panel_findandreplace_index = None

    self.subtitles_panel_findandreplace_toggle_button = QPushButton()  # It will be 'Find and replace'
    self.subtitles_panel_findandreplace_toggle_button.setObjectName('subtitles_panel_findandreplace_toggle_button')
    self.subtitles_panel_findandreplace_toggle_button.setProperty('class', 'button_dark')
    self.subtitles_panel_findandreplace_toggle_button.setProperty('borderless_right', 'true')
    self.subtitles_panel_findandreplace_toggle_button.setFixedWidth(23)
    self.subtitles_panel_findandreplace_toggle_button.clicked.connect(lambda: subtitles_panel_findandreplace_toggle_button_clicked(self))
    self.subtitles_panel_widget_buttons_vbox.addWidget(self.subtitles_panel_findandreplace_toggle_button)

    self.subtitles_panel_findandreplace_panel = QWidget(self)
    self.subtitles_panel_findandreplace_panel.setWindowFlags(Qt.Tool)
    self.subtitles_panel_findandreplace_panel.setObjectName('subtitles_panel_findandreplace_panel')
    self.subtitles_panel_findandreplace_panel.setLayout(QVBoxLayout())
    self.subtitles_panel_findandreplace_panel.layout().setSpacing(5)
    self.subtitles_panel_findandreplace_panel.layout().setContentsMargins(10, 10, 10, 10)
    self.subtitles_panel_findandreplace_panel.setVisible(False)

    self.subtitles_panel_findandreplace_find_line = QWidget(self)
    self.subtitles_panel_findandreplace_find_line.setLayout(QHBoxLayout())
    # self.subtitles_panel_findandreplace_find_line.setSizePolicy(QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Minimum))
    self.subtitles_panel_findandreplace_find_line.layout().setSpacing(0)
    self.subtitles_panel_findandreplace_find_line.layout().setContentsMargins(0, 0, 0, 0)
    self.subtitles_panel_findandreplace_find_line.layout().setSizeConstraint(QLayout.SetMaximumSize)

    self.subtitles_panel_findandreplace_findback_button = QPushButton()
    self.subtitles_panel_findandreplace_findback_button.setObjectName('subtitles_panel_findandreplace_findback_button')
    self.subtitles_panel_findandreplace_findback_button.setProperty('class', 'button_dark')
    self.subtitles_panel_findandreplace_findback_button.setProperty('borderless_right', True)
    self.subtitles_panel_findandreplace_findback_button.setSizePolicy(QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum))
    self.subtitles_panel_findandreplace_findback_button.clicked.connect(lambda: subtitles_panel_findandreplace_findback_field_clicked(self))
    self.subtitles_panel_findandreplace_find_line.layout().addWidget(self.subtitles_panel_findandreplace_findback_button, 0)

    self.subtitles_panel_findandreplace_find_field = QLineEdit()
    self.subtitles_panel_findandreplace_find_field.setObjectName('subtitles_panel_findandreplace_find_field')
    self.subtitles_panel_findandreplace_find_field.setProperty('borderless_right', True)
    self.subtitles_panel_findandreplace_find_field.setProperty('borderless_left', True)
    self.subtitles_panel_findandreplace_find_field.setSizePolicy(QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Minimum))
    self.subtitles_panel_findandreplace_find_field.setFixedWidth(150)
    self.subtitles_panel_findandreplace_find_field.textChanged.connect(lambda: subtitles_panel_findandreplace_find_field_textchanged(self))
    self.subtitles_panel_findandreplace_find_line.layout().addWidget(self.subtitles_panel_findandreplace_find_field, 1)

    self.subtitles_panel_findandreplace_findnext_button = QPushButton('FIND')
    self.subtitles_panel_findandreplace_findnext_button.setObjectName('subtitles_panel_findandreplace_findnext_button')
    self.subtitles_panel_findandreplace_findnext_button.setProperty('class', 'button_dark')
    self.subtitles_panel_findandreplace_findnext_button.setProperty('borderless_left', True)
    self.subtitles_panel_findandreplace_findnext_button.setSizePolicy(QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum))
    self.subtitles_panel_findandreplace_findnext_button.clicked.connect(lambda: subtitles_panel_findandreplace_findnext_field_clicked(self))
    self.subtitles_panel_findandreplace_find_line.layout().addWidget(self.subtitles_panel_findandreplace_findnext_button, 0)

    self.subtitles_panel_findandreplace_find_line.layout().addStretch()

    self.subtitles_panel_findandreplace_casesensitive = QPushButton()
    self.subtitles_panel_findandreplace_casesensitive.setObjectName('subtitles_panel_findandreplace_casesensitive')
    self.subtitles_panel_findandreplace_casesensitive.setProperty('class', 'button')
    self.subtitles_panel_findandreplace_casesensitive.setCheckable(True)
    self.subtitles_panel_findandreplace_casesensitive.clicked.connect(lambda: subtitles_panel_findandreplace_casesensitive_clicked(self))
    self.subtitles_panel_findandreplace_find_line.layout().addWidget(self.subtitles_panel_findandreplace_casesensitive, 0)

    self.subtitles_panel_findandreplace_panel.layout().addWidget(self.subtitles_panel_findandreplace_find_line)

    self.subtitles_panel_findandreplace_information_label = QLabel()
    self.subtitles_panel_findandreplace_information_label.setProperty('class', 'qlabel_for_buttons')
    self.subtitles_panel_findandreplace_panel.layout().addWidget(self.subtitles_panel_findandreplace_information_label)

    self.subtitles_panel_findandreplace_replace_line = QWidget(self)
    self.subtitles_panel_findandreplace_replace_line.setLayout(QHBoxLayout())
    self.subtitles_panel_findandreplace_replace_line.layout().setSpacing(0)
    self.subtitles_panel_findandreplace_replace_line.layout().setContentsMargins(0, 0, 0, 0)
    self.subtitles_panel_findandreplace_replace_line.layout().setSizeConstraint(QLayout.SetMaximumSize)

    self.subtitles_panel_findandreplace_replace_field = QLineEdit()
    self.subtitles_panel_findandreplace_replace_field.setObjectName('subtitles_panel_findandreplace_replace_field')
    self.subtitles_panel_findandreplace_replace_field.setProperty('borderless_right', True)
    self.subtitles_panel_findandreplace_replace_field.setSizePolicy(QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Minimum))
    self.subtitles_panel_findandreplace_replace_field.setFixedWidth(150)
    # self.subtitles_panel_findandreplace_replace_field.setFixedHeight(25)
    # self.subtitles_panel_findandreplace_replace_field.setVisible(False)
    self.subtitles_panel_findandreplace_replace_line.layout().addWidget(self.subtitles_panel_findandreplace_replace_field, 1)

    self.subtitles_panel_findandreplace_replaceandfindnext_button = QPushButton()
    self.subtitles_panel_findandreplace_replaceandfindnext_button.setProperty('class', 'button_dark')
    self.subtitles_panel_findandreplace_replaceandfindnext_button.setProperty('borderless_left', True)
    self.subtitles_panel_findandreplace_replaceandfindnext_button.setProperty('borderless_right', True)
    self.subtitles_panel_findandreplace_replaceandfindnext_button.setLayout(QHBoxLayout())
    self.subtitles_panel_findandreplace_replaceandfindnext_button.layout().setContentsMargins(0, 2, 4, 2)
    self.subtitles_panel_findandreplace_replaceandfindnext_button.layout().setSizeConstraint(QLayout.SetMinimumSize)
    self.subtitles_panel_findandreplace_replaceandfindnext_button.clicked.connect(lambda: subtitles_panel_findandreplace_replaceandfindnext_button_clicked(self))

    self.subtitles_panel_findandreplace_replace_button = QPushButton('REPLACE')
    self.subtitles_panel_findandreplace_replace_button.setProperty('class', 'button_dark')
    self.subtitles_panel_findandreplace_replace_button.setProperty('borderless_left', True)
    self.subtitles_panel_findandreplace_replace_button.setObjectName('subtitles_panel_findandreplace_replace_button')
    self.subtitles_panel_findandreplace_replace_button.clicked.connect(lambda: subtitles_panel_findandreplace_replace_button_clicked(self))
    self.subtitles_panel_findandreplace_replaceandfindnext_button.layout().addWidget(self.subtitles_panel_findandreplace_replace_button, 0)

    self.subtitles_panel_findandreplace_replaceandfindnext_button_label = QLabel('AND FIND NEXT')
    self.subtitles_panel_findandreplace_replaceandfindnext_button_label.setProperty('class', 'qlabel_for_buttons')
    self.subtitles_panel_findandreplace_replaceandfindnext_button.layout().addWidget(self.subtitles_panel_findandreplace_replaceandfindnext_button_label, 0)

    self.subtitles_panel_findandreplace_replace_line.layout().addWidget(self.subtitles_panel_findandreplace_replaceandfindnext_button, 0)

    self.subtitles_panel_findandreplace_replace_line.layout().addSpacing(1)

    self.subtitles_panel_findandreplace_replaceall_button = QPushButton('REPLACE ALL')
    self.subtitles_panel_findandreplace_replaceall_button.setProperty('class', 'button_dark')
    self.subtitles_panel_findandreplace_replaceall_button.setProperty('borderless_left', True)
    self.subtitles_panel_findandreplace_replaceall_button.clicked.connect(lambda: subtitles_panel_findandreplace_replaceall_button_clicked(self))
    self.subtitles_panel_findandreplace_replace_line.layout().addWidget(self.subtitles_panel_findandreplace_replaceall_button, 0)

    # self.subtitles_panel_findandreplace_replace_line.layout().addStretch()

    self.subtitles_panel_findandreplace_panel.layout().addWidget(self.subtitles_panel_findandreplace_replace_line)

    self.subtitles_panel_findandreplace_panel.setFixedHeight(self.subtitles_panel_findandreplace_panel.minimumSizeHint().height())

    self.subtitles_panel_toggle_button = QPushButton(parent=self)
    self.subtitles_panel_toggle_button.setFixedSize(QSize(22, 70))
    self.subtitles_panel_toggle_button.clicked.connect(lambda: subtitles_panel_toggle_button_clicked(self))
    self.subtitles_panel_toggle_button.setCheckable(True)
    self.subtitles_panel_toggle_button.setObjectName('subtitles_panel_toggle_button')
    self.subtitles_panel_toggle_button_animation = QPropertyAnimation(self.subtitles_panel_toggle_button, b'geometry')
    self.subtitles_panel_toggle_button_animation.setEasingCurve(QEasingCurve.OutCirc)

    self.subtitles_panel_widget.layout().addLayout(self.subtitles_panel_widget_buttons_vbox)

    update_subtitles_panel_widget_vision(self)


def resized(self):
    """Function to call when resizing subtitles list"""
    x = int(-((self.width() * self.subtitles_panel_width_proportion) - 15))
    if (self.subtitles_list or self.video_metadata) and not self.subtitles_panel_toggle_button.isChecked():
            x = 0
    self.subtitles_panel_widget.setGeometry(x, 0, int((self.width() * self.subtitles_panel_width_proportion) - 15), int(self.height()))

    # x = self.subtitles_panel_widget.x() + self.subtitles_panel_widget.width()
    # if (self.subtitles_list or self.video_metadata) and self.subtitles_panel_toggle_button.isChecked():
    #     x = self.global_panel_widget.x() + self.global_panel_widget.width() - self.subtitles_panel_toggle_button.width()
    # x -= self.subtitles_panel_toggle_button.width()

    self.subtitles_panel_toggle_button.move(self.global_panel_widget.x() + self.global_panel_widget.width() - self.subtitles_panel_toggle_button.width(), self.subtitles_panel_widget.y())
    subtitles_panel_widget_timeline.timeline_resized(self)


def subtitles_panel_toggle_button_clicked(self):
    """Function to call when clicking toggle button"""
    if self.subtitles_panel_toggle_button.isChecked():
        subtitles_panel_toggle_button_to_end(self)
        self.global_panel.show_global_panel(self)
        self.playercontrols.hide_playercontrols(self)
        hide(self)
    else:
        self.global_panel.hide_global_panel(self)
        self.playercontrols.show_playercontrols(self)
        show(self)


def update_topbar_status(self):
    # self.toppanel_format_label.setObjectName('toppanel_format_label')
    self.toppanel_format_label.setProperty('class', 'unsaved' if self.unsaved else 'saved')
    self.toppanel_format_label.setStyleSheet(self.toppanel_format_label.styleSheet())


def subtitles_panel_toggle_button_to_end(self):
    """Function to show subtitles list panel"""
    self.generate_effect(self.subtitles_panel_toggle_button_animation, 'geometry', 700, [self.subtitles_panel_toggle_button.x(), self.subtitles_panel_toggle_button.y(), self.subtitles_panel_toggle_button.width(), self.subtitles_panel_toggle_button.height()], [self.global_panel_widget.width() - 22, self.global_panel_widget.y(), self.subtitles_panel_toggle_button.width(), self.subtitles_panel_toggle_button.height()])


def update_subtitles_panel_widget_vision_content(self):
    if self.subtitles_panel_stackedwidgets.currentWidget() == self.subtitles_panel_simplelist_widget:
        subtitles_panel_widget_qlistwidget.update_subtitles_panel_qlistwidget(self)

    elif self.subtitles_panel_stackedwidgets.currentWidget() == self.subtitles_panel_markdown_widget:
        if not self.subtitles_panel_markdown_qtextedit.hasFocus():
            subtitles_panel_widget_markdown.update_subtitles_panel_markdown(self)

    elif self.subtitles_panel_stackedwidgets.currentWidget() == self.subtitles_panel_timeline_widget:
        subtitles_panel_widget_timeline.update_subtitles_panel_timeline(self)


def update_subtitles_panel_format_label(self):
    self.toppanel_format_label_text.setText(self.format_to_save)


def show(self):
    """Function to show subtitle list panel"""
    self.generate_effect(
        self.subtitles_panel_widget_animation,
        'geometry',
        700,
        [int(self.subtitles_panel_widget.x()), int(self.subtitles_panel_widget.y()), int(self.subtitles_panel_widget.width()), int(self.subtitles_panel_widget.height())],
        [0, int(self.subtitles_panel_widget.y()), int(self.subtitles_panel_widget.width()), int(self.subtitles_panel_widget.height())]
    )
    self.generate_effect(self.subtitles_panel_toggle_button_animation, 'geometry', 700, [self.subtitles_panel_toggle_button.x(), self.subtitles_panel_toggle_button.y(), self.subtitles_panel_toggle_button.width(), self.subtitles_panel_toggle_button.height()], [self.subtitles_panel_widget.width() - 22, self.subtitles_panel_widget.y(), self.subtitles_panel_toggle_button.width(), self.subtitles_panel_toggle_button.height()])
    self.global_panel.hide_global_panel(self)
    update_toppanel_subtitle_file_info_label(self)
    update_subtitles_panel_widget_vision_content(self)


def hide(self):
    """Function to hide subtitle list panel"""
    self.generate_effect(self.subtitles_panel_widget_animation, 'geometry', 700, [self.subtitles_panel_widget.x(), self.subtitles_panel_widget.y(), self.subtitles_panel_widget.width(), self.subtitles_panel_widget.height()], [-self.subtitles_panel_widget.width(), self.subtitles_panel_widget.y(), self.subtitles_panel_widget.width(), self.subtitles_panel_widget.height()])


def toppanel_save_button_clicked(self):
    """Function to call when save button on subtitles list panel is clicked"""
    actual_subtitle_file = False
    subtitle_format = self.settings['default_values'].get('subtitle_format', 'USF')
    if self.actual_subtitle_file:
        for formt in LIST_OF_SUPPORTED_SUBTITLE_EXTENSIONS:
            for ext in LIST_OF_SUPPORTED_SUBTITLE_EXTENSIONS[formt]['extensions']:
                if self.actual_subtitle_file.endswith(ext):
                    actual_subtitle_file = self.actual_subtitle_file
                    subtitle_format = formt
                    break

    if not actual_subtitle_file:
        suggested_path = os.path.dirname(self.video_metadata['filepath'])
        suggested_filename = os.path.basename(self.video_metadata['filepath']).rsplit('.', 1)[0] + '.' + LIST_OF_SUPPORTED_SUBTITLE_EXTENSIONS[subtitle_format]['extensions'][0]

        # save_formats = self.format_to_save + ' ' + LIST_OF_SUPPORTED_SUBTITLE_EXTENSIONS[self.format_to_save]['description'] + ' ({})'.format(" ".join(["*.{}".format(fo) for fo in LIST_OF_SUPPORTED_SUBTITLE_EXTENSIONS[self.format_to_save]['extensions']]))
        # for extformat in LIST_OF_SUPPORTED_SUBTITLE_EXTENSIONS:
        #     if not extformat == self.format_to_save:
        #         save_formats += ';;' + extformat + ' ' + LIST_OF_SUPPORTED_SUBTITLE_EXTENSIONS[extformat]['description'] + ' ({})'.format(" ".join(["*.{}".format(fo) for fo in LIST_OF_SUPPORTED_SUBTITLE_EXTENSIONS[extformat]['extensions']]))
        # suggested_name = os.path.basename(self.video_metadata['filepath']).rsplit('.', 1)[0]

        # tem que reportar o bug que não retorna o selectedFilter se o dialogo for nativo
        # filedialog = QFileDialog.getSaveFileName(parent=self, caption=self.tr('Select the subtitle file'), dir=os.path.join(suggested_path, suggested_name), filter=save_formats)

        # if filedialog[0] and filedialog[1]:
        #     filename = filedialog[0]
        #     exts = []
        #     for ext in filedialog[1].split('(', 1)[1].split(')', 1)[0].split('*'):
        #         if ext:
        #             exts.append(ext.strip())
        #     if not filename.endswith(tuple(exts)):
        #         filename += exts[0]
        #     if not self.format_to_save == filedialog[1].split(' ', 1)[0]:
        #         self.format_to_save = filedialog[1].split(' ', 1)[0]

        #     self.actual_subtitle_file = filename

        self.actual_subtitle_file = os.path.join(suggested_path, suggested_filename)

    if self.actual_subtitle_file:
        file_io.save_file(self.actual_subtitle_file, self.subtitles_list, subtitle_format, self.selected_language)
        if self.settings['default_values'].get('save_automatic_copy', False) and not subtitle_format == self.settings['default_values'].get('subtitle_format', 'USF'):
            file_io.save_file(self.actual_subtitle_file.rsplit('.', 1)[0] + '.usf', self.subtitles_list, self.settings['default_values'].get('subtitle_format', 'USF'), self.selected_language)
        self.format_usf_present = True
        update_subtitles_panel_format_label(self)
        update_toppanel_subtitle_file_info_label(self)
        self.unsaved = False

    update_topbar_status(self)


def toppanel_open_button_clicked(self, update_interface=True):
    """Function to call when open button on subtitles list panel is clicked"""
    if self.unsaved:
        save_message_box = QMessageBox(self)

        save_message_box.setWindowTitle(self.tr('Unsaved changes'))
        save_message_box.setText(
            self.tr('Do you want to save the changes you made on the subtitles?')
        )
        save_message_box.addButton(self.tr('Save'), QMessageBox.AcceptRole)
        save_message_box.addButton(self.tr("Don't save"), QMessageBox.RejectRole)
        ret = save_message_box.exec_()

        if ret == QMessageBox.AcceptRole:
            self.subttileslist.toppanel_save_button_clicked(self)

    file_io.open_filepath(self)
    update_topbar_status(self)


def subtitles_panel_findandreplace_toggle_button_clicked(self):
    self.subtitles_panel_findandreplace_panel.setVisible(True)
    subtitles_panel_findandreplace_find_field_textchanged(self)


def subtitles_panel_findandreplace_find_field_textchanged(self):
    self.subtitles_panel_findandreplace_findback_button.setEnabled(bool(self.subtitles_panel_findandreplace_find_field.text()))
    self.subtitles_panel_findandreplace_findnext_button.setEnabled(bool(self.subtitles_panel_findandreplace_find_field.text()))
    self.subtitles_panel_findandreplace_replaceandfindnext_button.setEnabled(bool(self.subtitles_panel_findandreplace_find_field.text()))
    self.subtitles_panel_findandreplace_replace_button.setEnabled(bool(self.subtitles_panel_findandreplace_find_field.text()))
    self.subtitles_panel_findandreplace_replaceall_button.setEnabled(bool(self.subtitles_panel_findandreplace_find_field.text()))

    subtitles_panel_findandreplace_perform_search(self)

    text = 'Type the text to find'
    if bool(self.subtitles_panel_findandreplace_find_field.text()):
        if self.subtitles_panel_findandreplace_list:
            text = 'Found {} matches'.format(len(self.subtitles_panel_findandreplace_list))
        else:
            text = 'No matches found for "{}"'.format(self.subtitles_panel_findandreplace_find_field.text())

    self.subtitles_panel_findandreplace_information_label.setText(text)


def subtitles_panel_findandreplace_replaceandfindnext_button_clicked(self):
    subtitles_panel_findandreplace_replace_button_clicked(self)
    subtitles_panel_findandreplace_findnext_field_clicked(self)


def subtitles_panel_findandreplace_replace_button_clicked(self):
    if self.selected_subtitle:
        subtitles.change_subtitle_text(subtitles=self.subtitles_list, selected_subtitle=self.selected_subtitle, text=self.selected_subtitle[2][:self.subtitles_panel_findandreplace_list[self.subtitles_panel_findandreplace_index][1]] + self.subtitles_panel_findandreplace_replace_field.text() + self.selected_subtitle[2][self.subtitles_panel_findandreplace_list[self.subtitles_panel_findandreplace_index][1] + self.subtitles_panel_findandreplace_list[self.subtitles_panel_findandreplace_index][2]:])
        self.unsaved = True
        update_topbar_status(self)

    ind = self.subtitles_panel_findandreplace_index
    subtitles_panel_findandreplace_find_field_textchanged(self)
    self.subtitles_panel_findandreplace_index = ind

    if self.subtitles_panel_stackedwidgets.currentWidget() == self.subtitles_panel_simplelist_widget:
        subtitles_panel_widget_qlistwidget.update_properties_widget(self)
    elif self.subtitles_panel_stackedwidgets.currentWidget() == self.subtitles_panel_markdown_widget:
        subtitles_panel_widget_markdown.update_subtitles_panel_markdown(self)
    elif self.subtitles_panel_stackedwidgets.currentWidget() == self.subtitles_panel_timeline_widget:
        subtitles_panel_widget_timeline.update_subtitles_panel_timeline(self)


def subtitles_panel_findandreplace_replaceall_button_clicked(self):
    while self.subtitles_panel_findandreplace_list:
        subtitles_panel_findandreplace_replaceandfindnext_button_clicked(self)
    subtitles_panel_findandreplace_update(self)


def subtitles_panel_findandreplace_casesensitive_clicked(self):
    subtitles_panel_findandreplace_find_field_textchanged(self)


def subtitles_panel_findandreplace_perform_search(self):
    self.subtitles_panel_findandreplace_list = []
    self.subtitles_panel_findandreplace_index = 0

    text_to_search = self.subtitles_panel_findandreplace_find_field.text() if self.subtitles_panel_findandreplace_casesensitive.isChecked() else self.subtitles_panel_findandreplace_find_field.text().lower()
    for subtitle in self.subtitles_list:
        if text_to_search in (subtitle[2] if self.subtitles_panel_findandreplace_casesensitive.isChecked() else subtitle[2].lower()):
            s = 0
            for i in range((subtitle[2] if self.subtitles_panel_findandreplace_casesensitive.isChecked() else subtitle[2].lower()).count(text_to_search)):
                self.subtitles_panel_findandreplace_list.append([self.subtitles_list.index(subtitle), subtitle[2].find(text_to_search, s), len(text_to_search)])
                s += subtitle[2].find(text_to_search, s) + len(text_to_search)


def subtitles_panel_findandreplace_findback_field_clicked(self):
    self.subtitles_panel_findandreplace_index -= 1
    if self.subtitles_panel_findandreplace_index < 0:
        self.subtitles_panel_findandreplace_index = len(self.subtitles_panel_findandreplace_list) - 1
    subtitles_panel_findandreplace_update(self)


def subtitles_panel_findandreplace_findnext_field_clicked(self):
    self.subtitles_panel_findandreplace_index += 1
    if self.subtitles_panel_findandreplace_index >= len(self.subtitles_panel_findandreplace_list):
        self.subtitles_panel_findandreplace_index = 0
    subtitles_panel_findandreplace_update(self)


def subtitles_panel_findandreplace_update(self):
    if self.subtitles_panel_findandreplace_list:
        self.selected_subtitle = self.subtitles_list[self.subtitles_panel_findandreplace_list[self.subtitles_panel_findandreplace_index][0]]

        if not self.selected_subtitle[0] < self.player_widget.position < self.selected_subtitle[0] + self.selected_subtitle[1]:
            self.player_widget.seek(self.selected_subtitle[0] + (self.selected_subtitle[1] * .5))
            timeline.update_scrollbar(self, position='middle')

        update_subtitles_panel_widget_vision_content(self)
        if self.subtitles_panel_stackedwidgets.currentWidget() == self.subtitles_panel_simplelist_widget:
            c = self.properties_textedit.textCursor()
            c.setPosition(self.subtitles_panel_findandreplace_list[self.subtitles_panel_findandreplace_index][1])
            c.setPosition(self.subtitles_panel_findandreplace_list[self.subtitles_panel_findandreplace_index][1] + self.subtitles_panel_findandreplace_list[self.subtitles_panel_findandreplace_index][2], QTextCursor.KeepAnchor)
            self.properties_textedit.setTextCursor(c)
        elif self.subtitles_panel_stackedwidgets.currentWidget() == self.subtitles_panel_markdown_widget:
            subtitles_panel_widget_markdown.update_subtitles_panel_markdown(self, selection=[self.subtitles_panel_findandreplace_list[self.subtitles_panel_findandreplace_index][1], self.subtitles_panel_findandreplace_list[self.subtitles_panel_findandreplace_index][2]])
        elif self.subtitles_panel_stackedwidgets.currentWidget() == self.subtitles_panel_timeline_widget:
            self.subtitles_panel_timeline_widget_timeline.show_editing_widgets = True
            self.subtitles_panel_timeline_widget_timeline.update_editing_widgets()
            c = self.subtitles_panel_timeline_widget_timeline.text_qtextedit.textCursor()
            c.setPosition(self.subtitles_panel_findandreplace_list[self.subtitles_panel_findandreplace_index][1])
            c.setPosition(self.subtitles_panel_findandreplace_list[self.subtitles_panel_findandreplace_index][1] + self.subtitles_panel_findandreplace_list[self.subtitles_panel_findandreplace_index][2], QTextCursor.KeepAnchor)
            self.subtitles_panel_timeline_widget_timeline.text_qtextedit.setTextCursor(c)
            subtitles_panel_widget_timeline.update_scrollbar(self, position='middle')


def update_toppanel_subtitle_file_info_label(self):
    """Function to update top information on subtitles list panel"""
    text = self.tr('Actual video does not have saved subtitle file.')
    if self.actual_subtitle_file:
        text = '<b><small>' + self.tr('Actual project:').upper() + '</small></b><br><big>' + os.path.basename(self.actual_subtitle_file) + '</big>'
    self.toppanel_subtitle_file_info_label.setText(text)


def update_subtitles_panel_widget_vision(self, vision='list'):
    if vision == 'list':
        self.subtitles_panel_stackedwidgets.setCurrentWidget(self.subtitles_panel_simplelist_widget)
        self.subtitles_panel_widget_button_list.setEnabled(False)
    else:
        self.subtitles_panel_widget_button_list.setEnabled(True)
        self.subtitles_panel_widget_button_list.setChecked(False)

    if vision == 'markdown':
        self.subtitles_panel_stackedwidgets.setCurrentWidget(self.subtitles_panel_markdown_widget)
        self.subtitles_panel_widget_button_markdown.setEnabled(False)
    else:
        self.subtitles_panel_widget_button_markdown.setEnabled(True)
        self.subtitles_panel_widget_button_markdown.setChecked(False)

    if vision == 'timeline':
        self.subtitles_panel_stackedwidgets.setCurrentWidget(self.subtitles_panel_timeline_widget)
        self.subtitles_panel_widget_button_timeline.setEnabled(False)
        subtitles_panel_widget_timeline.timeline_resized(self)
    else:
        self.subtitles_panel_widget_button_timeline.setEnabled(True)
        self.subtitles_panel_widget_button_timeline.setChecked(False)

    update_subtitles_panel_widget_vision_content(self)



"""Subtitles Video panel

"""
import subprocess

from PySide6.QtWidgets import QPushButton, QWidget
from PySide6.QtCore import QThread, Signal

from subtitld.modules.paths import STARTUPINFO
from subtitld.modules import utils
from subtitld.interface import global_panel


class ThreadGeneratedBurnedVideo(QThread):
    """Thread to generate burned video"""
    response = Signal(str)
    commands = []

    def run(self):
        """Run function of thread to generate burned video"""
        if self.commands:
            proc = subprocess.Popen(self.commands, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, universal_newlines=True, startupinfo=STARTUPINFO, bufsize=4096)
            number_of_steps = 0.001
            current_step = 0.0
            while proc.poll() is None:
                output = proc.stdout.readline()
                if 'Duration: ' in output:
                    duration = int(utils.convert_ffmpeg_timecode_to_seconds(output.split('Duration: ', 1)[1].split(',', 1)[0]))
                    if duration > number_of_steps:
                        number_of_steps = duration
                if output[:6] == 'frame=':
                    current_step = int(utils.convert_ffmpeg_timecode_to_seconds(output.split('time=', 1)[1].split(' ', 1)[0]))

                self.response.emit(str(current_step) + '|' + str(number_of_steps))
            self.response.emit('end')


def load_menu(self):
    """Function to load subtitles panel widgets"""
    self.global_panel_voiceover_menu_button = QPushButton('Voice over')
    self.global_panel_voiceover_menu_button.setCheckable(True)
    self.global_panel_voiceover_menu_button.setProperty('class', 'global_panel_menu')
    self.global_panel_voiceover_menu_button.clicked.connect(lambda: global_panel_menu_changed(self))
    self.global_panel_menu.layout().addWidget(self.global_panel_voiceover_menu_button)


def global_panel_menu_changed(self):
    self.global_panel_voiceover_menu_button.setEnabled(False)
    global_panel.global_panel_menu_changed(self, self.global_panel_voiceover_menu_button, self.global_panel_voiceover_content)


def load_widgets(self):
    """Function to load subtitles panel widgets"""

    self.global_panel_voiceover_content = QWidget()

    self.global_subtitlesvideo_voiceover_button = QPushButton(self.tr('Voiceover').upper(), parent=self.global_panel_voiceover_content)
    self.global_subtitlesvideo_voiceover_button.setProperty('class', 'button')
    self.global_subtitlesvideo_voiceover_button.clicked.connect(lambda: global_subtitlesvideo_autovoiceover_button_clicked(self))


def global_subtitlesvideo_autovoiceover_button_clicked(self):
    """Function to auto voiceover subtitles"""
    None
    # speech_config = SpeechConfig(subscription="", region="southcentralus")
    # speech_config.set_speech_synthesis_output_format(SpeechSynthesisOutputFormat["Riff24Khz16BitMonoPcm"])

    # audio_from_video = AudioSegment.from_file(self.video_metadata['filepath'])
    # final_audio = AudioSegment.empty()

    # audio_config = AudioOutputConfig(filename=os.path.join(path_tmp, 'voiceover.wav'))

    # audio_pieces = []
    # parser = 0
    # first = True
    # for subtitle in self.subtitles_list:
    #     # print(subtitle)

    #     ssml_content = '<speak version="1.0" xmlns="http://www.w3.org/2001/10/synthesis" xmlns:mstts="https://www.w3.org/2001/mstts" xml:lang="en-US"><voice name="pt-BR-AntonioNeural">'
    #     ssml_content += subtitle[2] + '</voice></speak>'

    #     synthesizer = SpeechSynthesizer(speech_config=speech_config, audio_config=audio_config)

    #     synthesizer.speak_ssml_async(ssml_content)
    #     audio_pieces.append([AudioSegment.from_file(os.path.join(path_tmp, 'voiceover.wav')), int(subtitle[0]*1000)])

    #     piece1 = audio_from_video[parser:int(subtitle[0]*1000)]
    #     if not first:
    #         piece1 = piece1.fade(from_gain=-25.0, start=0, duration=4000)
    #     piece1 = piece1.fade(to_gain=-25.0, end=int(piece1.duration_seconds*1000), duration=1200)
    #     final_audio += piece1
    #     parser = int(subtitle[0]*1000)

    #     piece2 = audio_from_video[parser:parser+int(subtitle[1]*1000)]
    #     piece2 = piece2 - 25.0
    #     final_audio += piece2
    #     parser += int(subtitle[1]*1000)
    #     first = False

    # final_piece = audio_from_video[parser:]
    # final_piece = final_piece.fade(from_gain=-25.0, start=0, duration=5000)
    # final_audio += final_piece

    # #new_audio = sum(audio_pieces)

    # for piece in audio_pieces:
    #     final_audio = final_audio.overlay(piece[0], position=piece[1])

    # final_audio.export(os.path.join(path_tmp, 'final_voiceover.wav'), format='wav')

    # subprocess.call(['ffmpeg', '-i', self.video_metadata['filepath'], '-i' , os.path.join(path_tmp, 'final_voiceover.wav'), '-c:v', 'copy', '-y', '-map', '0:v:0', '-map', '1:a:0', self.video_metadata['filepath'].rsplit('.', 1)[0] + '_voiceover.' + self.video_metadata['filepath'].rsplit('.', 1)[-1]])

    # # list_of_final_audiofiles = []
    # #     audio_config = AudioOutputConfig(filename="file.wav")
    # #     synthesizer = SpeechSynthesizer(speech_config=speech_config, audio_config=None)
    # #     synthesizer.speak_ssml_async(open('teste.ssml').read()))
    # #     File "<stdin>", line 1
    # #         synthesizer.speak_ssml_async(open('teste.ssml').read()))
    # #                                                             ^
    # #     SyntaxError: unmatched ')'
    # #

    # #     language = LANGUAGE_DICT_LIST[self.global_subtitlesvideo_autosync_lang_combobox.currentText()].split('-')[0]
    # #     translator = Translator(service_urls=['translate.googleapis.com'])

    # #         subtitle[2] = translator.translate(subtitle[2], dest=language).text

    # #     update_widgets(self)



import os
import datetime
import html
import hashlib
from docx import Document

from PySide6.QtWidgets import QFileDialog
from PySide6.QtCore import QThread, Signal

# from scenedetect.video_manager import VideoManager
# from scenedetect.scene_manager import SceneManager
# from scenedetect.stats_manager import StatsManager
# from scenedetect.detectors import ContentDetector

import pycaption
from pycaption.exceptions import CaptionReadSyntaxError, CaptionReadNoCaptions
import chardet
import pysubs2
# from cleantext import clean
from subtitld import timecode  # , captionstransformer

from subtitld.modules import waveform
from subtitld.modules import usf
from subtitld.modules.paths import LIST_OF_SUPPORTED_SUBTITLE_EXTENSIONS, LIST_OF_SUPPORTED_VIDEO_EXTENSIONS, REAL_PATH_HOME

list_of_supported_subtitle_extensions = []
for exttype in LIST_OF_SUPPORTED_SUBTITLE_EXTENSIONS:
    for ext in LIST_OF_SUPPORTED_SUBTITLE_EXTENSIONS[exttype]['extensions']:
        list_of_supported_subtitle_extensions.append(ext)


class ThreadExtractSceneTimePositions(QThread):
    """Thread to extract time positions of scenes"""
    command = Signal(list)
    filepath = ''

    def run(self):
        """Run function of extract time positions thread"""
        if self.filepath:
            result = []
            try:
                video_manager = VideoManager([self.filepath])
                stats_manager = StatsManager()
                scene_manager = SceneManager(stats_manager)
                scene_manager.add_detector(ContentDetector())
                base_timecode = video_manager.get_base_timecode()
                try:
                    video_manager.set_downscale_factor()
                    video_manager.start()
                    scene_manager.detect_scenes(frame_source=video_manager, show_progress=False)
                    scene_list = scene_manager.get_scene_list(base_timecode)
                    for _, scene in enumerate(scene_list):
                        result.append(scene[0].get_seconds())
                finally:
                    video_manager.release()
            except Exception:
                pass
            self.command.emit(result)


class ThreadExtractWaveform(QThread):
    """Thread to extract waveform"""
    command = Signal(list)
    filepath = ''
    audio = ''
    duration = ''
    width = ''
    height = ''

    def run(self):
        """Run function of thread to extract waveform"""
        if self.filepath:
            # result = waveform.ffmpeg_load_audio(filepath=self.filepath)
            result = waveform.ffms2_load_audio(filepath=self.filepath)
            self.command.emit(result)


class ThreadGenerateHashOfVideo(QThread):
    """Thread to extract time positions of scenes"""
    response = Signal(list)
    filepath = ''

    def run(self):
        if self.filepath and os.path.isfile(self.filepath):
            md5_object = hashlib.md5()
            block_size = 128 * md5_object.block_size
            file_object = open(self.filepath, 'rb')
            chunk = file_object.read(block_size)
            while chunk:
                md5_object.update(chunk)
                chunk = file_object.read(block_size)

            md5_hash = md5_object.hexdigest()

            self.response.emit([self.filepath, md5_hash])


def load(self):
    """Load thread objects"""
    def thread_extract_waveform_ended(command):
        print('audio ffms2 ended')
        self.video_metadata['audio'] = command
        self.timeline.zoom_update_waveform(self)
        self.videoinfo_label.setText(self.tr('Audio ffmpeg_extract_subtitleed'))

    self.thread_extract_waveform = ThreadExtractWaveform(self)
    self.thread_extract_waveform.command.connect(thread_extract_waveform_ended)

    def thread_extract_waveform_ended2(command):
        if not command[0] in self.video_metadata['waveform']:
            self.video_metadata['waveform'][command[0]] = {'qimages': []}
        self.video_metadata['waveform'][command[0]]['qimages'].append(command[1])
        self.timeline_widget.update()

    self.thread_extract_waveform2 = waveform.ThreadExtractWaveform2(self)
    self.thread_extract_waveform2.command.connect(thread_extract_waveform_ended2)

    # def thread_extract_scene_time_positions_ended(command):
    #     self.video_metadata['scenes'] = command

    # self.thread_extract_scene_time_positions = ThreadExtractSceneTimePositions(self)
    # self.thread_extract_scene_time_positions.command.connect(thread_extract_scene_time_positions_ended)

    def thread_generate_hash_of_video(response):
        if self.video_metadata.get('filepath', '') == response[0] and 'hash' not in self.video_metadata:
            self.video_metadata['hash'] = response[1]
            if self.actual_subtitle_file in self.settings['recent_files']:
                self.settings['recent_files'][self.actual_subtitle_file]['video_hash'] = self.video_metadata['hash']

    self.thread_generate_hash_of_video = ThreadGenerateHashOfVideo(self)
    self.thread_generate_hash_of_video.response.connect(thread_generate_hash_of_video)


def open_filepath(self, files_to_open=False, update_interface=False):
    """Open subtitle or video and performs some checks"""
    supported_subtitle_files = self.tr('Subtitle files') + ' ({})'.format(" ".join(["*.{}".format(fo) for fo in list_of_supported_subtitle_extensions]))
    supported_video_files = self.tr('Video files') + ' ({})'.format(" ".join(["*{}".format(fo) for fo in LIST_OF_SUPPORTED_VIDEO_EXTENSIONS]))

    if not files_to_open:
        files_to_open = [QFileDialog.getOpenFileName(parent=self, caption=self.tr('Select the video or subtitle file'), dir=REAL_PATH_HOME, filter=supported_subtitle_files + ';;' + supported_video_files)[0]]

    for filepath in files_to_open:
        if os.path.isfile(filepath):
            if not self.subtitles_list and filepath.lower().endswith(tuple(list_of_supported_subtitle_extensions)):
                self.subtitles_list, self.format_to_save = process_subtitles_file(filepath)
                self.actual_subtitle_file = filepath

            elif not self.video_metadata and filepath.lower().endswith(LIST_OF_SUPPORTED_VIDEO_EXTENSIONS):
                self.video_metadata = process_video_file(filepath)

            if not self.video_metadata and self.subtitles_list:
                for filename in os.listdir(os.path.dirname(filepath)):
                    if filename.rsplit('.', 1)[0] == os.path.basename(filepath).rsplit('.', 1)[0] and filename.endswith(LIST_OF_SUPPORTED_VIDEO_EXTENSIONS):
                        self.video_metadata = process_video_file(os.path.join(os.path.dirname(filepath), filename))
                        break

            elif self.video_metadata and not self.subtitles_list:
                for filename in os.listdir(os.path.dirname(filepath)):
                    if filename.rsplit('.', 1)[0] == os.path.basename(filepath).rsplit('.', 1)[0] and filename.endswith(tuple(list_of_supported_subtitle_extensions)):
                        self.subtitles_list, self.format_to_save = process_subtitles_file(os.path.join(os.path.dirname(filepath), filename))
                        self.actual_subtitle_file = os.path.join(os.path.dirname(filepath), filename)
                        break

            if not self.video_metadata:
                filepath = QFileDialog.getOpenFileName(parent=self.parent(), caption=self.tr('Select the video file'), dir=REAL_PATH_HOME, filter=supported_video_files)[0]
                if filepath and os.path.isfile(filepath) and filepath.lower().endswith(LIST_OF_SUPPORTED_VIDEO_EXTENSIONS):
                    self.video_metadata = process_video_file(filepath)

    if self.video_metadata:
        self.actual_video_file = self.video_metadata['filepath']
        if self.video_metadata['audio_is_present']:
            self.thread_extract_waveform.filepath = self.video_metadata['filepath']
            self.thread_extract_waveform.start()
            self.videoinfo_label.setText(self.tr('Extracting audio...'))

        self.player_widget.loadfile(self.video_metadata['filepath'])
        if self.actual_subtitle_file:
            self.thread_generate_hash_of_video.filepath = self.video_metadata['filepath']
            self.thread_generate_hash_of_video.start()
            if self.actual_subtitle_file in self.settings['recent_files']:
                self.player_widget.seek(self.settings['recent_files'][self.actual_subtitle_file].get('last_position', 0))

        if not self.actual_subtitle_file:
            if self.video_metadata.get('subtitles', ''):
                self.subtitles_list, self.format_to_save = process_subtitles_file(self.video_metadata['subtitles'])
        self.subtitles_panel.update_subtitles_panel_widget_vision_content(self)
        self.subtitles_panel.update_topbar_status(self)
        self.settings['recent_files'][self.actual_subtitle_file] = {
            'last_opened': datetime.datetime.now().strftime("%Y%m%d%H%M%S"),
            'video_filepath': self.video_metadata['filepath']
        }
        self.autosave_timer.start()

        if update_interface:
            self.timeline.update_timeline(self)
            self.startscreen.hide(self)
            self.playercontrols.show(self)
            self.subtitles_panel.show(self)
            self.global_panel.hide_global_panel(self)
            player_qrect = self.player.resize_player_widget(self, just_get_qrect=True)
            original_qrect = [self.start_screen_thumbnail_background.x(), self.start_screen_thumbnail_background.y(), self.start_screen_thumbnail_background.width(), self.start_screen_thumbnail_background.height()]
            self.generate_effect(self.start_screen_thumbnail_background_transparency_animation, 'opacity', 200, 1.0, 0.0)
            self.generate_effect(self.player_border_animation, 'geometry', 700, original_qrect, player_qrect)
            self.generate_effect(self.player_border_transparency_animation, 'opacity', 700, 0.5, 1.0)

    if os.path.isfile(os.path.join(os.path.dirname(self.actual_subtitle_file), os.path.basename(self.actual_subtitle_file).rsplit('.', 1)[0] + '.usf')):
        self.format_usf_present = True
    else:
        self.format_usf_present = False

    self.subtitles_panel.update_subtitles_panel_format_label(self)


def process_subtitles_file(subtitle_file=False, subtitle_format='SRT'):
    """Definition to process subtitle file. It returns a dict with the subtitles."""
    final_subtitles = []

    if subtitle_file and os.path.isfile(subtitle_file):
        if subtitle_file.lower().endswith(('.srt')):
            enc = chardet.detect(open(subtitle_file, 'rb').read())['encoding']
            with open(subtitle_file, mode='rb') as srt_file:
                srt_content = srt_file.read().decode(enc, 'ignore')

                if ' -> ' in srt_content:
                    srt_content = srt_content.replace(' -> ', ' --> ')

                srt_reader = pycaption.SRTReader().read(srt_content)
                languages = srt_reader.get_languages()
                language = languages[0]
                captions = srt_reader.get_captions(language)
                for caption in captions:
                    final_subtitles.append([caption.start / 1000000, (caption.end / 1000000) - caption.start / 1000000, caption.get_text()])

        elif subtitle_file.lower().endswith(('.vtt', '.webvtt')):
            subtitle_format = 'VTT'
            with open(subtitle_file, encoding='utf-8') as vtt_file:
                try:
                    vtt_reader = pycaption.WebVTTReader().read(vtt_file.read())
                    languages = vtt_reader.get_languages()
                    language = languages[0]
                    captions = vtt_reader.get_captions(language)
                    for caption in captions:
                        final_subtitles.append([caption.start / 1000000, (caption.end / 1000000) - caption.start / 1000000, caption.get_text()])
                except CaptionReadSyntaxError:
                    with open(subtitle_file, encoding='utf-8') as fileobj:
                        subfile = pysubs2.SSAFile.from_string(fileobj.read())

                    for event in subfile.events:
                        start = event.start / 1000.0
                        duration = event.duration / 1000.0
                        text = event.plaintext
                        final_subtitles.append([start, duration, text])
                except CaptionReadNoCaptions:
                    pass

                    # error_message = QMessageBox()
                    # error_message.setWindowTitle(self.tr('There is a problem with this file and can not be opened.'))

        elif subtitle_file.lower().endswith(('.ttml', '.dfxp', '.xml', '.itt')):
            # if subtitle_file.lower().endswith(('.xml')) and '<transcript>' in open(subtitle_file).read():
            #     subtitle_format = 'XML'
            #     with open(subtitle_file, encoding='utf-8') as xml_file:
            #         captions = captionstransformer.transcript.Reader(xml_file).read()
            #         for caption in captions:
            #             final_subtitles.append([(caption.start - datetime.datetime(1900, 1, 1)).total_seconds(), caption.duration.total_seconds(), html.unescape(caption.text)])
            # else:
            if '<tt ' in open(subtitle_file).read():
                subtitle_format = 'TTML'
            else:
                subtitle_format = 'DFXP'

            with open(subtitle_file, encoding='utf-8') as dfxp_file:
                dfxp_reader = pycaption.DFXPReader().read(dfxp_file.read())
                languages = dfxp_reader.get_languages()
                language = languages[0]
                captions = dfxp_reader.get_captions(language)
                for caption in captions:
                    final_subtitles.append([caption.start / 1000000, (caption.end / 1000000) - caption.start / 1000000, caption.get_text()])

        elif subtitle_file.lower().endswith(('.smi', '.sami')):
            subtitle_format = 'SAMI'
            with open(subtitle_file, encoding='utf-8') as sami_file:
                sami_reader = pycaption.SAMIReader().read(sami_file.read())
                languages = sami_reader.get_languages()
                language = languages[0]
                captions = sami_reader.get_captions(language)
                for caption in captions:
                    final_subtitles.append([caption.start / 1000000, (caption.end / 1000000) - caption.start / 1000000, caption.get_text()])

        # elif subtitle_file.lower().endswith(('.sbv')):
        #     subtitle_format = 'SBV'
        #     with open(subtitle_file, encoding='utf-8') as sbv_file:
        #         from captionstransformer.sbv import Reader
        #         captions = Reader(sbv_file).read()
        #         for caption in captions:
        #             final_subtitles.append([(caption.start - datetime.datetime(1900, 1, 1)).total_seconds(), caption.duration.total_seconds(), caption.text.strip()])

        elif subtitle_file.lower().endswith(('.ass', '.ssa', '.sub')):
            if subtitle_file.lower().endswith(('.ass', '.ssa')):
                subtitle_format = 'ASS'
                with open(subtitle_file, encoding='utf-8') as fileobj:
                    subfile = pysubs2.SSAFile.from_string(fileobj.read())
            elif subtitle_file.lower().endswith(('.sub')):
                subtitle_format = 'SUB'
                enc = chardet.detect(open(subtitle_file, 'rb').read())['encoding']
                subfile = pysubs2.SSAFile.from_string(open(subtitle_file, mode='rb').read().decode(enc, 'ignore'))

            for event in subfile.events:
                start = event.start / 1000.0
                duration = event.duration / 1000.0
                text = event.plaintext
                final_subtitles.append([start, duration, text])

        elif subtitle_file.lower().endswith(('.scc')):
            subtitle_format = 'SCC'

            with open(subtitle_file, encoding='utf-8') as scc_file:
                scc_reader = pycaption.SCCReader().read(scc_file.read())
                languages = scc_reader.get_languages()
                language = languages[0]
                captions = scc_reader.get_captions(language)
                for caption in captions:
                    final_subtitles.append([caption.start / 1000000, (caption.end / 1000000) - caption.start / 1000000, caption.get_text()])




        elif subtitle_file.lower().endswith(('.usf')):
            subtitle_format = 'USF'
            final_subtitles = usf.USFReader().read(open(subtitle_file).read())

    return final_subtitles, subtitle_format


def process_video_file(video_file=False):
    """Function to process video file. It returns a dict with video properties."""
    video_metadata = {}
    json_result = waveform.ffmpeg_load_metadata(video_file)
    video_metadata['audio'] = False
    video_metadata['audio_is_present'] = False
    video_metadata['waveform'] = {}
    video_metadata['duration'] = float(json_result.get('format', {}).get('duration', '0.01'))
    for stream in json_result.get('streams', []):
        if stream.get('codec_type', '') == 'video' and not stream.get('codec_name', 'png') in ['png', 'mjpeg']:
            video_metadata['width'] = int(stream.get('width', 640))
            video_metadata['height'] = int(stream.get('height', 480))
            video_metadata['framerate'] = int(stream.get('r_frame_rate', '1/30').split('/', 1)[0]) / int(stream.get('r_frame_rate', '1/30').split('/', 1)[-1])
        elif stream.get('codec_type', '') in ['subtitle'] and not video_metadata.get('subtitles', False):
            video_metadata['subtitles'] = waveform.ffmpeg_extract_subtitle(video_file, stream.get('index', 2))
            # TODO: select what language if multiple embedded subtitles
        elif stream.get('codec_type', '') in ['audio']:
            video_metadata['audio_is_present'] = True
    video_metadata['filepath'] = video_file
    video_metadata['scenes'] = []

    return video_metadata


def import_file(filename=False, subtitle_format=False):  # , fit_to_length=False, length=.01, distribute_fixed_duration=False):
    """Function to import file into the subtitle project."""
    final_subtitles = []
    if filename:
        if filename.lower().endswith(('.txt')):
            subtitle_format = 'TXT'
            with open(filename) as txt_file:
                # clean("some input",
                #     fix_unicode=True,               # fix various unicode errors
                #     to_ascii=True,                  # transliterate to closest ASCII representation
                #     lower=True,                     # lowercase text
                #     no_line_breaks=False,           # fully strip line breaks as opposed to only normalizing them
                #     no_urls=False,                  # replace all URLs with a special token
                #     no_emails=False,                # replace all email addresses with a special token
                #     no_phone_numbers=False,         # replace all phone numbers with a special token
                #     no_numbers=False,               # replace all numbers with a special token
                #     no_digits=False,                # replace all digits with a special token
                #     no_currency_symbols=False,      # replace all currency symbols with a special token
                #     no_punct=False,                 # fully remove punctuation
                #     replace_with_url="<URL>",
                #     replace_with_email="<EMAIL>",
                #     replace_with_phone_number="<PHONE>",
                #     replace_with_number="<NUMBER>",
                # replace_with_digit="0",
                # replace_with_currency_symbol="<CUR>",
                # lang="en"                       # set to 'de' for German special handling
                # )

                # txt_content = clean(txt_file.read())
                txt_content = txt_file.read()
                pos = 0.0
                for phrase in txt_content.split('. '):
                    final_subtitles.append([pos, 5.0, phrase + '.'])
                    pos += 5.0

        elif filename.lower().endswith(('.srt')):
            subtitle_format = 'SRT'
            final_subtitles += process_subtitles_file(subtitle_file=filename, subtitle_format=subtitle_format)[0]

        elif filename.lower().endswith(('.docx')):
            subtitle_format = 'DOCX'
            txt_content = ''

            doc = Document(filename)
            for par in doc.paragraphs:
                txt_content += par.text

            pos = 0.0
            for phrase in txt_content.split('. '):
                final_subtitles.append([pos, 5.0, phrase + '.'])
                pos += 5.0

            final_subtitles += process_subtitles_file(subtitle_file=filename, subtitle_format=subtitle_format)[0]

    return final_subtitles, subtitle_format


def export_file(filename=False, subtitles_list=False, export_format='TXT', options=False):
    """Function to export file. A filepath and a subtitle dict is given."""
    if subtitles_list and filename:
        if export_format in ['.txt']:
            final_txt = ''
            for sub in subtitles_list:
                final_txt += sub[2].replace('\n', ' ') + ' '
            if options:
                if options.get('new_line', False):
                    final_txt = final_txt.replace('. ', '.\n')
                    final_txt = final_txt.replace('! ', '!\n')
                    final_txt = final_txt.replace('? ', '?\n')
            with open(filename, mode='w', encoding='utf-8') as txt_file:
                txt_file.write(final_txt)
        elif export_format in ['.kdenlive']:
            final_xml = '''<?xml version='1.0' encoding='utf-8'?><mlt LC_NUMERIC="C" producer="main_bin" version="6.26.1" root="/home/jonata"><profile frame_rate_num="25" sample_aspect_num="1" display_aspect_den="9" colorspace="709" progressive="1" description="HD 1080p 25 fps" display_aspect_num="16" frame_rate_den="1" width="1920" height="1080" sample_aspect_den="1"/>'''

            i = 0
            for sub in subtitles_list:
                final_xml += '''<producer id="producer{i}" in="{zerotime}" out="{out}">
                                <property name="length">{length}</property>
                                <property name="eof">pause</property>
                                <property name="resource"/>
                                <property name="progressive">1</property>
                                <property name="aspect_ratio">1</property>
                                <property name="seekable">1</property>
                                <property name="mlt_service">kdenlivetitle</property>
                                <property name="kdenlive:duration">125</property>
                                <property name="kdenlive:clipname">{clipname}</property>
                                <property name="xmldata">&lt;kdenlivetitle duration="125" LC_NUMERIC="C" width="1920" height="1080" out="124"> &lt;item type="QGraphicsTextItem" z-index="0"> &lt;position x="784" y="910"> &lt;transform>1,0,0,0,1,0,0,0,1&lt;/transform> &lt;/position>
                                &lt;content shadow="0;#64000000;3;3;3" font-underline="0" box-height="62" font-outline-color="0,0,0,255" font="Ubuntu" letter-spacing="0" font-pixel-size="54" font-italic="0" typewriter="0;2;1;0;0" alignment="1" font-weight="50" font-outline="0"
                                box-width="351.719" font-color="255,255,255,255">{content}&lt;/content> &lt;/item> &lt;startviewport rect="0,0,1920,1080"/> &lt;endviewport rect="0,0,1920,1080"/> &lt;background color="0,0,0,0"/> &lt;/kdenlivetitle>
                                </property>
                                <property name="kdenlive:folderid">-1</property>
                                <property name="kdenlive:id">{id}</property>
                                <property name="force_reload">0</property>
                                <property name="meta.media.width">1920</property>
                                <property name="meta.media.height">1080</property>
                            </producer>'''.format(i=i, id=i + 2, length=int(sub[1] * 25), clipname='Subtitle {i}'.format(i=i), content=sub[2], zerotime=str(timecode.Timecode('1000', start_seconds=0.001, fractional=True)), out=str(timecode.Timecode('1000', start_seconds=sub[1], fractional=True)))
                i += 1

            final_xml += '''<playlist id="main_bin">
                            <property name="kdenlive:docproperties.activeTrack">2</property>
                            <property name="kdenlive:docproperties.audioChannels">2</property>
                            <property name="kdenlive:docproperties.audioTarget">-1</property>
                            <property name="kdenlive:docproperties.disablepreview">0</property>
                            <property name="kdenlive:docproperties.documentid">1621801540856</property>
                            <property name="kdenlive:docproperties.enableTimelineZone">0</property>
                            <property name="kdenlive:docproperties.enableexternalproxy">0</property>
                            <property name="kdenlive:docproperties.enableproxy">0</property>
                            <property name="kdenlive:docproperties.externalproxyparams">../Sub;;S03.MP4;../Clip;;.MXF</property>
                            <property name="kdenlive:docproperties.generateimageproxy">0</property>
                            <property name="kdenlive:docproperties.generateproxy">0</property>
                            <property name="kdenlive:docproperties.groups">[ ]
                            </property>
                            <property name="kdenlive:docproperties.kdenliveversion">21.04.0</property>
                            <property name="kdenlive:docproperties.position">372</property>
                            <property name="kdenlive:docproperties.previewextension"/>
                            <property name="kdenlive:docproperties.previewparameters"/>
                            <property name="kdenlive:docproperties.profile">atsc_1080p_25</property>
                            <property name="kdenlive:docproperties.proxyextension"/>
                            <property name="kdenlive:docproperties.proxyimageminsize">2000</property>
                            <property name="kdenlive:docproperties.proxyimagesize">800</property>
                            <property name="kdenlive:docproperties.proxyminsize">1000</property>
                            <property name="kdenlive:docproperties.proxyparams"/>
                            <property name="kdenlive:docproperties.scrollPos">0</property>
                            <property name="kdenlive:docproperties.seekOffset">30000</property>
                            <property name="kdenlive:docproperties.version">1</property>
                            <property name="kdenlive:docproperties.verticalzoom">1</property>
                            <property name="kdenlive:docproperties.videoTarget">-1</property>
                            <property name="kdenlive:docproperties.zonein">0</property>
                            <property name="kdenlive:docproperties.zoneout">75</property>
                            <property name="kdenlive:docproperties.zoom">8</property>
                            <property name="kdenlive:expandedFolders"/>
                            <property name="kdenlive:documentnotes"/>
                            <property name="xml_retain">1</property>\n'''

            i = 0
            for sub in subtitles_list:
                final_xml += '''<entry producer="producer{i}" in="{zerotime}" out="{out}"/>\n'''.format(i=i, zerotime=str(timecode.Timecode('1000', start_seconds=0.001, fractional=True)), out=str(timecode.Timecode('1000', start_seconds=sub[1], fractional=True)))
                i += 1

            final_xml += '''</playlist>
                            <producer id="black_track" in="00:00:00.000" out="00:20:12.120">
                                <property name="length">2147483647</property>
                                <property name="eof">continue</property>
                                <property name="resource">black</property>
                                <property name="aspect_ratio">1</property>
                                <property name="mlt_service">color</property>
                                <property name="mlt_image_format">rgb24a</property>
                                <property name="set.test_audio">0</property>
                            </producer>
                            <playlist id="playlist0">
                                <property name="kdenlive:audio_track">1</property>
                            </playlist>
                            <playlist id="playlist1"/>
                            <tractor id="tractor0" in="00:00:00.000">
                                <property name="kdenlive:audio_track">1</property>
                                <property name="kdenlive:trackheight">69</property>
                                <property name="kdenlive:timeline_active">1</property>
                                <property name="kdenlive:collapsed">0</property>
                                <property name="kdenlive:thumbs_format"/>
                                <property name="kdenlive:audio_rec"/>
                                <track hide="video" producer="playlist0"/>
                                <track hide="video" producer="playlist1"/>
                            </tractor>
                            <playlist id="playlist2">
                                <property name="kdenlive:audio_track">1</property>
                            </playlist>
                            <playlist id="playlist3"/>
                            <tractor id="tractor1" in="00:00:00.000">
                                <property name="kdenlive:audio_track">1</property>
                                <property name="kdenlive:trackheight">69</property>
                                <property name="kdenlive:timeline_active">1</property>
                                <property name="kdenlive:collapsed">0</property>
                                <property name="kdenlive:thumbs_format"/>
                                <property name="kdenlive:audio_rec"/>
                                <track hide="video" producer="playlist2"/>
                                <track hide="video" producer="playlist3"/>
                            </tractor>
                            <playlist id="playlist4"/>
                            <playlist id="playlist5"/>
                            <tractor id="tractor2" in="00:00:00.000" out="00:00:12.080">
                                <property name="kdenlive:trackheight">69</property>
                                <property name="kdenlive:timeline_active">1</property>
                                <property name="kdenlive:collapsed">0</property>
                                <property name="kdenlive:thumbs_format"/>
                                <property name="kdenlive:audio_rec"/>
                                <track hide="audio" producer="playlist4"/>
                                <track producer="playlist5"/>
                            </tractor>
                            <playlist id="playlist6">'''
            i = 0
            last_intime = 0
            for sub in subtitles_list:
                last_intime = sub[0] - last_intime
                if last_intime:
                    final_xml += '''
                                    <blank length="{intime}"/>'''.format(intime=str(timecode.Timecode('1000', start_seconds=last_intime, fractional=True)))
                final_xml += '''
                                <entry producer="producer{i}" in="{zerotime}" out="{out}">
                                    <property name="kdenlive:id">{id}</property>
                                </entry>'''.format(i=i, id=i + 2, zerotime=str(timecode.Timecode('1000', start_seconds=0.001, fractional=True)), out=str(timecode.Timecode('1000', start_seconds=sub[1], fractional=True)))
                last_intime = sub[0] + sub[1]
                i += 1

            final_xml += '''
                            </playlist>
                            <playlist id="playlist7"/>
                            <tractor id="tractor3" in="00:00:00.000">
                                <property name="kdenlive:trackheight">69</property>
                                <property name="kdenlive:timeline_active">1</property>
                                <property name="kdenlive:collapsed">0</property>
                                <property name="kdenlive:thumbs_format"/>
                                <property name="kdenlive:audio_rec"/>
                                <track hide="audio" producer="playlist6"/>
                                <track producer="playlist7"/>
                            </tractor>
                            <tractor id="tractor4" global_feed="1" in="00:00:00.000" out="00:20:12.120">
                                <track producer="black_track"/>
                                <track producer="tractor0"/>
                                <track producer="tractor1"/>
                                <track producer="tractor2"/>
                                <track producer="tractor3"/>
                            </tractor>
                            </mlt>'''

            with open(filename, mode='w', encoding='utf-8') as txt_file:
                txt_file.write(final_xml)


def save_file(final_file, subtitles_list, subtitle_format='SRT', language='en'):
    """Function to save the subtitle project. A subtitles dict and the format is given."""
    if subtitles_list:
        # if not final_file.lower().endswith('.' + format.lower()):
        #     final_file += '.' + format.lower()

        if subtitle_format in ['SRT', 'DFXP', 'TTML', 'SAMI', 'SCC', 'VTT']:
            captions = pycaption.CaptionList()
            for sub in subtitles_list:
                # skip extra blank lines
                nodes = [pycaption.CaptionNode.create_text(sub[2])]
                caption = pycaption.Caption(start=sub[0] * 1000000, end=(sub[0] + sub[1]) * 1000000, nodes=nodes)
                captions.append(caption)
            caption_set = pycaption.CaptionSet({language: captions})

            if subtitle_format == 'SRT':
                open(final_file, mode='w', encoding='utf-8').write(pycaption.SRTWriter().write(caption_set))
            elif subtitle_format in ['DFXP', 'TTML']:
                open(final_file, mode='w', encoding='utf-8').write(pycaption.DFXPWriter().write(caption_set))
            elif subtitle_format == 'SAMI':
                open(final_file, mode='w', encoding='utf-8').write(pycaption.SAMIWriter().write(caption_set))
            elif subtitle_format == 'SCC':
                open(final_file, mode='w', encoding='utf-8').write(pycaption.SCCWriter().write(caption_set))
            elif subtitle_format == 'VTT':
                open(final_file, mode='w', encoding='utf-8').write(pycaption.WebVTTWriter().write(caption_set))

        elif subtitle_format in ['ASS', 'SBV', 'XML', 'SUB']:
            if subtitle_format in ['ASS', 'SUB']:
                assfile = pysubs2.SSAFile()
                index = 0
                for sub in reversed(sorted(subtitles_list)):
                    assfile.insert(index, pysubs2.SSAEvent(start=int(sub[0] * 1000), end=int((sub[0] * 1000) + (sub[1] * 1000)), text=sub[2].replace('\n', ' ')))
                if subtitle_format == 'SUB':
                    assfile.save(final_file, subtitle_format='microdvd')
                else:
                    assfile.save(final_file)
            # else:
            #     if subtitle_format == 'SBV':
            #         from captionstransformer.sbv import Writer
            #     elif subtitle_format == 'XML':
            #         from captionstransformer.transcript import Writer
            #     writer = Writer(open(final_file, mode='w', encoding='utf-8'))
            #     captions = []
            #     for cap in subtitles_list:
            #         caption = captionstransformer.core.Caption()
            #         caption.start = captionstransformer.core.get_date(second=int(cap[0] // 1), millisecond=int((cap[0] % 1) * 1000))
            #         caption.duration = captionstransformer.core.get_date(second=int(cap[1] // 1), millisecond=int((cap[1] % 1) * 1000)) - captionstransformer.core.get_date()
            #         caption.text = cap[2]
            #         captions.append(caption)
            #     writer.set_captions(captions)
            #     writer.write()

        elif subtitle_format in ['USF']:
            open(final_file, mode='w', encoding='utf-8').write(usf.USFWriter().write(subtitles_list))



"""Subtitles Video panel

"""
from PySide6.QtWidgets import QPushButton, QWidget, QTableWidget, QAbstractItemView, QLineEdit, QTableWidgetItem, QHeaderView, QVBoxLayout, QHBoxLayout, QLabel, QGridLayout, QSizePolicy
from PySide6.QtCore import QEvent, Qt
from PySide6.QtGui import QKeySequence
from subtitld.modules import shortcuts

from subtitld.modules.shortcuts import shortcuts_dict, default_shortcuts_dict
from subtitld.interface import global_panel


class global_panel_keyboardshortcut_qlineedit(QLineEdit):
    """Class to reimplement QLineEdit in order to get shorkeys selection"""
    def __init__(widget, *args, parent=None):
        super(global_panel_keyboardshortcut_qlineedit, widget).__init__(*args)
        widget.setLayout(QHBoxLayout())
        widget.setAlignment(Qt.AlignCenter)
        widget.setReadOnly(True)
        widget.setSizePolicy(QSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.Preferred))
        widget.layout().setContentsMargins(0, 0, 0, 0)
        widget.layout().setSpacing(5)
        widget.command = ''

    def update_shortcuts(widget):
        while widget.layout().count():
            child = widget.layout().takeAt(0).widget()
            if child:
                child.setParent(None)
                child.deleteLater()

        command_string = widget.command

        widget.layout().addStretch()

        if not command_string:
            qlabel = QLabel('Press the shorkey combination for this command' if widget.isEnabled() else 'No shorkey for this command.')
            qlabel.setProperty('class', 'units_label')
            qlabel.setAlignment(Qt.AlignCenter)
            widget.layout().addWidget(qlabel, 0, Qt.AlignCenter | Qt.AlignBottom)
        else:
            if command_string == '+':
                command_string = '＋'
            for key in command_string.replace('++', '+＋').split('+'):
                if key:
                    button = QPushButton(key)
                    button.setProperty('class', 'keyboard_key')
                    widget.layout().addWidget(button, 0, Qt.AlignCenter | Qt.AlignBottom)

        widget.layout().addStretch()

    def keyPressEvent(widget, event):
        widget.setReadOnly(False)
        if event.type() == QEvent.KeyPress:
            key = event.key()

            if key == Qt.Key_unknown:
                return

            if key in [Qt.Key_Control, Qt.Key_Shift, Qt.Key_Alt, Qt.Key_Meta]:
                return

            modifiers = event.modifiers()
            # keyText = event.text()

            if modifiers & Qt.ShiftModifier:
                key += Qt.SHIFT
            if modifiers & Qt.ControlModifier:
                key += Qt.CTRL
            if modifiers & Qt.AltModifier:
                key += Qt.ALT
            if modifiers & Qt.MetaModifier:
                key += Qt.META
            widget.command = QKeySequence(key).toString()
            widget.update_shortcuts()

        widget.setReadOnly(True)


def load_menu(self):
    """Function to load subtitles panel widgets"""
    self.global_panel_keyboardshortcuts_menu_button = QPushButton('Keyboard shortcuts')
    self.global_panel_keyboardshortcuts_menu_button.setCheckable(True)
    self.global_panel_keyboardshortcuts_menu_button.setProperty('class', 'global_panel_menu')
    self.global_panel_keyboardshortcuts_menu_button.clicked.connect(lambda: global_panel_menu_changed(self))
    self.global_panel_menu.layout().addWidget(self.global_panel_keyboardshortcuts_menu_button)


def global_panel_menu_changed(self):
    self.global_panel_keyboardshortcuts_menu_button.setEnabled(False)
    global_panel.global_panel_menu_changed(self, self.global_panel_keyboardshortcuts_menu_button, self.global_panel_keyboardshortcuts_content)


def load_widgets(self):
    """Function to load subtitles panel widgets"""

    self.global_panel_keyboardshortcuts_content = QWidget()
    self.global_panel_keyboardshortcuts_content.setLayout(QGridLayout())
    self.global_panel_keyboardshortcuts_content.layout().setContentsMargins(0, 0, 0, 0)

    self.global_panel_tabwidget_shortkeys_table = QTableWidget()
    self.global_panel_tabwidget_shortkeys_table.setObjectName('global_panel_tabwidget_shortkeys_table')
    self.global_panel_tabwidget_shortkeys_table.setColumnCount(3)
    self.global_panel_tabwidget_shortkeys_table.verticalHeader().setVisible(False)
    self.global_panel_tabwidget_shortkeys_table.setColumnHidden(1, True)
    self.global_panel_tabwidget_shortkeys_table.setEditTriggers(QTableWidget.EditTrigger.NoEditTriggers)
    self.global_panel_tabwidget_shortkeys_table.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
    self.global_panel_tabwidget_shortkeys_table.horizontalHeader().setHighlightSections(False)
    self.global_panel_tabwidget_shortkeys_table.horizontalHeader().setDefaultAlignment(Qt.AlignLeft | Qt.AlignVCenter)
    self.global_panel_tabwidget_shortkeys_table.setShowGrid(False)
    self.global_panel_tabwidget_shortkeys_table.setSelectionBehavior(QAbstractItemView.SelectRows)
    self.global_panel_tabwidget_shortkeys_table.clicked.connect(lambda: global_panel_tabwidget_shortkeys_table_clicked(self))
    self.global_panel_keyboardshortcuts_content.layout().addWidget(self.global_panel_tabwidget_shortkeys_table, 0, 0, 3, 1)

    self.global_panel_tabwidget_shortkeys_details = QVBoxLayout()

    self.global_panel_tabwidget_shortkeys_details_warning = QLabel('Select a command')
    self.global_panel_tabwidget_shortkeys_details_warning.setProperty('class', 'units_label')
    self.global_panel_tabwidget_shortkeys_details.addWidget(self.global_panel_tabwidget_shortkeys_details_warning, 0, Qt.AlignCenter)

    self.global_panel_keyboardshortcut_qlineedit = global_panel_keyboardshortcut_qlineedit()
    self.global_panel_keyboardshortcut_qlineedit.setObjectName('global_panel_keyboardshortcut_qlineedit')
    self.global_panel_tabwidget_shortkeys_details.addWidget(self.global_panel_keyboardshortcut_qlineedit)

    self.global_panel_keyboardshortcut_buttons = QWidget()
    self.global_panel_keyboardshortcut_buttons.setLayout(QHBoxLayout())
    self.global_panel_keyboardshortcut_buttons.layout().setContentsMargins(0, 10, 0, 0)
    self.global_panel_keyboardshortcut_buttons.layout().setSpacing(5)

    self.global_panel_keyboardshortcut_change_button = QPushButton('Change')
    self.global_panel_keyboardshortcut_change_button.setProperty('class', 'subbutton2_dark')
    self.global_panel_keyboardshortcut_change_button.clicked.connect(lambda: global_panel_keyboardshortcut_change_button_clicked(self))
    self.global_panel_keyboardshortcut_buttons.layout().addWidget(self.global_panel_keyboardshortcut_change_button)

    self.global_panel_keyboardshortcut_confirm_button = QPushButton('Confirm')
    self.global_panel_keyboardshortcut_confirm_button.setProperty('class', 'subbutton2_dark')
    self.global_panel_keyboardshortcut_confirm_button.clicked.connect(lambda: global_panel_keyboardshortcut_confirm_button_clicked(self))
    self.global_panel_keyboardshortcut_buttons.layout().addWidget(self.global_panel_keyboardshortcut_confirm_button)

    self.global_panel_keyboardshortcut_cancel_button = QPushButton('Cancel')
    self.global_panel_keyboardshortcut_cancel_button.setProperty('class', 'subbutton2_dark')
    self.global_panel_keyboardshortcut_cancel_button.clicked.connect(lambda: global_panel_keyboardshortcut_cancel_button_clicked(self))
    self.global_panel_keyboardshortcut_buttons.layout().addWidget(self.global_panel_keyboardshortcut_cancel_button)

    self.global_panel_keyboardshortcut_clear_button = QPushButton('Clear')
    self.global_panel_keyboardshortcut_clear_button.setProperty('class', 'subbutton2_dark')
    self.global_panel_keyboardshortcut_clear_button.clicked.connect(lambda: global_panel_keyboardshortcut_clear_button_clicked(self))
    self.global_panel_keyboardshortcut_buttons.layout().addWidget(self.global_panel_keyboardshortcut_clear_button)

    self.global_panel_tabwidget_shortkeys_details.addWidget(self.global_panel_keyboardshortcut_buttons, 0, Qt.AlignCenter | Qt.AlignTop)

    self.global_panel_keyboardshortcuts_content.layout().addLayout(self.global_panel_tabwidget_shortkeys_details, 3, 0, 1, 1)

    self.global_panel_content_stacked_widgets.addWidget(self.global_panel_keyboardshortcuts_content)

    global_panel_tabwidget_shortkeys_table_update(self)


def global_panel_tabwidget_shortkeys_table_update(self):
    """Function to update subtitlesvideo panel shorkeys table"""
    self.global_panel_tabwidget_shortkeys_table.clear()
    self.global_panel_tabwidget_shortkeys_table.setRowCount(len(shortcuts_dict))
    self.global_panel_tabwidget_shortkeys_table.setHorizontalHeaderLabels(['Command', 'internal_command', 'Shortkeys'])
    # inverted_shortcuts_dict = {value: key for key, value in shortcuts_dict.items()}
    i = 0
    for item in shortcuts_dict:
        item_name = QTableWidgetItem(shortcuts_dict[item])
        self.global_panel_tabwidget_shortkeys_table.setItem(i, 0, item_name)
        item_name = QTableWidgetItem(item)
        self.global_panel_tabwidget_shortkeys_table.setItem(i, 1, item_name)
        item_name = QTableWidgetItem(self.settings['shortcuts'].get(item, default_shortcuts_dict.get(item, ['']))[0])
        self.global_panel_tabwidget_shortkeys_table.setItem(i, 2, item_name)
        i += 1
    update_global_panel_tabwidget_shortkeys_details(self)


def global_panel_tabwidget_shortkeys_table_clicked(self):
    update_global_panel_tabwidget_shortkeys_details(self)
    global_panel_keyboardshortcut_cancel_button_clicked(self)


def update_global_panel_tabwidget_shortkeys_details(self):
    self.global_panel_tabwidget_shortkeys_details_warning.setVisible(not bool(self.global_panel_tabwidget_shortkeys_table.currentItem()))
    self.global_panel_keyboardshortcut_qlineedit.setVisible(bool(self.global_panel_tabwidget_shortkeys_table.currentItem()))
    self.global_panel_keyboardshortcut_buttons.setVisible(bool(self.global_panel_tabwidget_shortkeys_table.currentItem()))

    if self.global_panel_tabwidget_shortkeys_table.currentItem():
        self.global_panel_keyboardshortcut_qlineedit.command = self.global_panel_tabwidget_shortkeys_table.item(self.global_panel_tabwidget_shortkeys_table.currentRow(), 2).text()
        self.global_panel_keyboardshortcut_qlineedit.update_shortcuts()


def global_panel_keyboardshortcut_change_button_clicked(self):
    self.global_panel_keyboardshortcut_confirm_button.setVisible(True)
    self.global_panel_keyboardshortcut_cancel_button.setVisible(True)
    self.global_panel_keyboardshortcut_change_button.setVisible(False)
    self.global_panel_keyboardshortcut_clear_button.setVisible(False)

    self.global_panel_keyboardshortcut_qlineedit.setEnabled(True)
    self.global_panel_keyboardshortcut_qlineedit.command = ''
    self.global_panel_keyboardshortcut_qlineedit.update_shortcuts()
    shortcuts.disable_actions(self)
    self.global_panel_keyboardshortcut_qlineedit.setFocus()


def global_panel_keyboardshortcut_confirm_button_clicked(self):
    command = self.global_panel_tabwidget_shortkeys_table.item(self.global_panel_tabwidget_shortkeys_table.currentRow(), 1).text()
    self.settings['shortcuts'][command] = [self.global_panel_keyboardshortcut_qlineedit.command]

    global_panel_keyboardshortcut_cancel_button_clicked(self)

    selected_row = self.global_panel_tabwidget_shortkeys_table.currentRow()
    global_panel_tabwidget_shortkeys_table_update(self)
    self.global_panel_tabwidget_shortkeys_table.setCurrentCell(selected_row, 0)
    global_panel_tabwidget_shortkeys_table_clicked(self)

    shortcuts.enable_actions(self)


def global_panel_keyboardshortcut_clear_button_clicked(self):
    command = self.global_panel_tabwidget_shortkeys_table.item(self.global_panel_tabwidget_shortkeys_table.currentRow(), 1).text()
    self.settings['shortcuts'][command] = ['']

    global_panel_keyboardshortcut_cancel_button_clicked(self)

    selected_row = self.global_panel_tabwidget_shortkeys_table.currentRow()
    global_panel_tabwidget_shortkeys_table_update(self)
    self.global_panel_tabwidget_shortkeys_table.setCurrentCell(selected_row, 0)
    global_panel_tabwidget_shortkeys_table_clicked(self)


def global_panel_keyboardshortcut_cancel_button_clicked(self):
    self.global_panel_keyboardshortcut_confirm_button.setVisible(False)
    self.global_panel_keyboardshortcut_cancel_button.setVisible(False)
    self.global_panel_keyboardshortcut_change_button.setVisible(True)
    self.global_panel_keyboardshortcut_clear_button.setVisible(True)
    self.global_panel_keyboardshortcut_qlineedit.setEnabled(False)

    update_global_panel_tabwidget_shortkeys_details(self)



"""Subtitles Video panel

"""

import os
import subprocess
import json

import speech_recognition as sr

from PySide6.QtWidgets import QComboBox, QPushButton, QWidget, QMessageBox, QVBoxLayout, QTabWidget, QHBoxLayout, QLabel, QGroupBox, QProgressBar
from PySide6.QtCore import QThread, Signal, Qt, QSize, QRect, QMargins
from PySide6.QtGui import QPainter, QPen, QColor, QFont
from subtitld.interface import global_panel

from subtitld.modules.paths import LANGUAGE_DICT_LIST, STARTUPINFO, path_tmp, FFMPEG_EXECUTABLE
from subtitld.modules import file_io
from subtitld.modules import utils
from subtitld import autosub

LANGUAGE_DESCRIPTIONS = LANGUAGE_DICT_LIST.keys()


class global_panel_transcription_transcript_thread(QThread):
    """Class of qtread to get waveform data"""
    result = Signal(str)
    text = Signal(str)

    def __init__(self):
        super().__init__()
        self.metadata = {}
        self.selected_language = False

    def run(self):
        final_text = ''
        total_steps = int(self.metadata['duration'] / 60)

        actual_split = 0
        while actual_split < self.metadata['duration']:
            if self.metadata and self.selected_language:
                self.result.emit('{}/{}'.format(int(actual_split / 60), total_steps))
                subprocess.run(
                    [
                        FFMPEG_EXECUTABLE,
                        '-y',
                        '-i',
                        self.metadata['filepath'],
                        '-ss', str(actual_split),
                        '-t', '60',
                        os.path.join(path_tmp, 'transcribe.wav')
                    ]
                )

                r = sr.Recognizer()
                with sr.AudioFile(os.path.join(path_tmp, 'transcribe.wav')) as source:
                    audio = r.record(source)

                language = LANGUAGE_DICT_LIST[self.selected_language].split('-')[0]

                try:
                    final_text += r.recognize_google(audio, language=language)
                except sr.UnknownValueError:
                    print("Google Speech Recognition could not understand audio")
                except sr.RequestError as e:
                    print("Could not request results from Google Speech Recognition service; {0}".format(e))

                actual_split += 60

        if final_text:
            self.text.emit(final_text)


class global_panel_transcription_transcript_preview(QLabel):
    def __init__(widget):
        super().__init__()
        widget.text = ''
        widget.division_type = 'equal'

    def update_type(widget, slice_type):
        widget.division_type = slice_type
        widget.update()

    def update_text(widget, text):
        widget.text = text
        widget.update()

    def paintEvent(widget, event):
        """Function for paintEvent of Timeline"""
        painter = QPainter(widget)
        painter.setRenderHint(QPainter.Antialiasing)

        main_qrect = QRect(
            0,
            0,
            widget.width(),
            widget.height()
        )

        main_qrect -= QMargins(
            10,
            1,
            10,
            1
        )

        painter.setPen(QPen(QColor('#33304251'), 1, Qt.SolidLine))
        painter.setBrush(QColor('#ffffff'))
        painter.drawRoundedRect(main_qrect, 2, 2)

        text_qrect = main_qrect - QMargins(
            8,
            8,
            8,
            8
        )

        painter.setFont(QFont('Ubuntu', 5))
        painter.setBrush(Qt.NoBrush)
        painter.setPen(QColor('#ff304251'))

        if widget.text:
            painter.drawText(text_qrect, Qt.AlignLeft | Qt.AlignTop | Qt.TextWordWrap, widget.text)
            painter.drawText(text_qrect, Qt.AlignLeft | Qt.AlignTop | Qt.TextWordWrap, widget.text)

            painter.setBrush(QColor('#0f1519'))
            painter.setPen(Qt.NoPen)

            timeline_qrect = QRect(
                0,
                (widget.height() - 30) * .75,
                widget.width(),
                34,
            )

            painter.drawRoundedRect(timeline_qrect, 2, 2)

            timeline_qrect -= QMargins(10, 10, -10, 10)

            painter.setPen(QColor('#ff6a7483'))
            painter.setBrush(QColor('#ccb8cee0'))

            if widget.division_type == 'equal':
                ns = 5
                for s in range(ns):
                    painter.setPen(QPen(QColor('#ff6a7483'), 1, Qt.SolidLine))
                    srect = timeline_qrect - QMargins(s * (timeline_qrect.width() / ns), 0, ((ns - 1 - s) * (timeline_qrect.width() / ns) + 1), 0)
                    painter.drawRoundedRect(srect, 2, 2)
                    painter.setPen(QPen(QColor('#ff6a7483'), 5, Qt.SolidLine, Qt.RoundCap))
                    srect -= QMargins(5, 2, 5, 0)
                    painter.drawLine(srect.left(), srect.center().y(), srect.right(), srect.center().y())
            elif widget.division_type == 'phrase':
                ns = 5 if len(widget.text.split('. ')) > 4 else len(widget.text.split('. '))
                tl = len('.'.join(widget.text.split('. ')[:ns]))
                x = 0
                for s in range(ns):
                    painter.setPen(QPen(QColor('#ff6a7483'), 1, Qt.SolidLine))
                    cl = (len(widget.text.split('. ')[s]) / tl) * timeline_qrect.width()
                    srect = timeline_qrect - QMargins(x, 0, timeline_qrect.width() - x - cl + 1, 0)
                    painter.drawRoundedRect(srect, 2, 2)
                    painter.setPen(QPen(QColor('#ff6a7483'), 5, Qt.SolidLine, Qt.RoundCap))
                    srect -= QMargins(5, 2, 5, 0)
                    painter.drawLine(srect.left(), srect.center().y(), srect.right(), srect.center().y())
                    x += cl
            elif widget.division_type == 'words':
                wlist = widget.text.split(' ')
                while('' in wlist):
                    wlist.remove('')
                ns = 15 if len(wlist) > 14 else len(wlist)
                tl = len(' '.join(wlist[:ns])) + (3 * ns)
                x = 0
                for s in range(ns):
                    painter.setPen(QPen(QColor('#ff6a7483'), 1, Qt.SolidLine))
                    cl = ((len(wlist[s]) + 3) / tl) * (timeline_qrect.width() + 50)
                    srect = timeline_qrect - QMargins(x, 0, timeline_qrect.width() - x - cl + 1, 0)
                    painter.drawRoundedRect(srect, 2, 2)
                    painter.setPen(QPen(QColor('#ff6a7483'), 5, Qt.SolidLine, Qt.RoundCap))
                    srect -= QMargins(5, 2, 5, 0)
                    painter.drawLine(srect.left(), srect.center().y(), srect.right(), srect.center().y())
                    x += cl
        else:
            painter.drawText(text_qrect, Qt.AlignCenter | Qt.TextWordWrap, 'No transcription.')

        painter.end()
        event.accept()


class ThreadGeneratedBurnedVideo(QThread):
    """Thread to generate burned video"""
    response = Signal(str)
    commands = []

    def run(self):
        """Run function of thread to generate burned video"""
        if self.commands:
            proc = subprocess.Popen(self.commands, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, universal_newlines=True, startupinfo=STARTUPINFO, bufsize=4096)
            number_of_steps = 0.001
            current_step = 0.0
            while proc.poll() is None:
                # for output in proc.stdout.read().decode().split('\n'):
                output = proc.stdout.readline()
                if 'Duration: ' in output:
                    duration = int(utils.convert_ffmpeg_timecode_to_seconds(output.split('Duration: ', 1)[1].split(',', 1)[0]))
                    if duration > number_of_steps:
                        number_of_steps = duration
                if output[:6] == 'frame=':
                    current_step = int(utils.convert_ffmpeg_timecode_to_seconds(output.split('time=', 1)[1].split(' ', 1)[0]))

                self.response.emit(str(current_step) + '|' + str(number_of_steps))
            self.response.emit('end')


def load_menu(self):
    """Function to load subtitles panel widgets"""
    self.global_panel_transcription_menu_button = QPushButton('Transcription')
    self.global_panel_transcription_menu_button.setCheckable(True)
    self.global_panel_transcription_menu_button.setProperty('class', 'global_panel_menu')
    self.global_panel_transcription_menu_button.clicked.connect(lambda: global_panel_menu_changed(self))
    self.global_panel_menu.layout().addWidget(self.global_panel_transcription_menu_button)


def global_panel_menu_changed(self):
    self.global_panel_transcription_menu_button.setEnabled(False)
    global_panel.global_panel_menu_changed(self, self.global_panel_transcription_menu_button, self.global_panel_transcription_content)


def load_widgets(self):
    """Function to load subtitles panel widgets"""

    self.global_panel_transcription_content = QWidget()
    self.global_panel_transcription_content.setLayout(QVBoxLayout())
    self.global_panel_transcription_content.layout().setContentsMargins(0, 0, 0, 0)
    self.global_panel_transcription_content.layout().setSpacing(10)

    self.global_panel_transcription_language_combobox = QComboBox()
    self.global_panel_transcription_language_combobox.setProperty('class', 'button')
    self.global_panel_transcription_language_combobox.addItems(LANGUAGE_DESCRIPTIONS)
    self.global_panel_transcription_content.layout().addWidget(self.global_panel_transcription_language_combobox, 0, Qt.AlignLeft)

    self.global_panel_transcription_tabwidget = QTabWidget()

    self.global_panel_transcription_autosubtitle_widget = QWidget()
    self.global_panel_transcription_autosubtitle_widget.setLayout(QVBoxLayout())
    self.global_panel_transcription_autosubtitle_widget.layout().setContentsMargins(10, 10, 10, 10)
    self.global_panel_transcription_autosubtitle_widget.layout().setSpacing(20)

    self.global_panel_transcription_autosubtitle_groupbox = QGroupBox('Google Web Speech Public API')
    self.global_panel_transcription_autosubtitle_groupbox.setLayout(QHBoxLayout())
    self.global_panel_transcription_autosubtitle_groupbox.layout().setContentsMargins(10, 10, 10, 10)
    self.global_panel_transcription_autosubtitle_groupbox.layout().setSpacing(5)

    self.global_panel_transcription_autosubtitle_button = QPushButton('Auto Subtitle')
    self.global_panel_transcription_autosubtitle_button.setProperty('class', 'button')
    self.global_panel_transcription_autosubtitle_button.clicked.connect(lambda: global_panel_transcription_autosubtitle_button_clicked(self))
    self.global_panel_transcription_autosubtitle_groupbox.layout().addWidget(self.global_panel_transcription_autosubtitle_button)

    self.global_panel_transcription_autosubtitle_label = QLabel('This will overwrite all of the subtitles on the project.')
    self.global_panel_transcription_autosubtitle_label.setProperty('class', 'units_label')
    self.global_panel_transcription_autosubtitle_groupbox.layout().addWidget(self.global_panel_transcription_autosubtitle_label)

    self.global_panel_transcription_autosubtitle_groupbox.layout().addStretch()

    self.global_panel_transcription_autosubtitle_widget.layout().addWidget(self.global_panel_transcription_autosubtitle_groupbox)

    self.global_panel_transcription_autosubtitle_widget.layout().addStretch()

    self.global_panel_transcription_tabwidget.addTab(self.global_panel_transcription_autosubtitle_widget, 'Autosubtitle')

    self.global_panel_transcription_transcript_widget = QWidget()
    self.global_panel_transcription_transcript_widget.setLayout(QHBoxLayout())
    self.global_panel_transcription_transcript_widget.layout().setContentsMargins(10, 10, 10, 10)
    self.global_panel_transcription_transcript_widget.layout().setSpacing(20)

    self.global_panel_transcription_transcript_column = QVBoxLayout()
    self.global_panel_transcription_transcript_column.setContentsMargins(0, 0, 0, 0)

    self.global_panel_transcription_transcript_groupbox = QGroupBox('Google Web Speech Public API')
    self.global_panel_transcription_transcript_groupbox.setLayout(QHBoxLayout())
    self.global_panel_transcription_transcript_groupbox.layout().setContentsMargins(10, 10, 10, 10)
    self.global_panel_transcription_transcript_groupbox.layout().setSpacing(5)

    def global_panel_transcription_transcript_thread_text_emmited(result):
        self.global_panel_transcription_transcript_preview.update_text(text=result)
        self.global_panel_transcription_transcript_button.setVisible(True)
        self.global_panel_transcription_transcript_progress.setVisible(False)
        self.global_panel_transcription_transcript_apply_transcript_button.setVisible(True)

    def global_panel_transcription_transcript_thread_result_emmited(result):
        if '/' in result:
            self.global_panel_transcription_transcript_button.setVisible(False)
            self.global_panel_transcription_transcript_progress.setVisible(True)
            self.global_panel_transcription_transcript_progress.setMaximum(int(result.split('/')[1]))
            self.global_panel_transcription_transcript_progress.setValue(int(result.split('/')[0]))

    self.global_panel_transcription_transcript_thread = global_panel_transcription_transcript_thread()
    self.global_panel_transcription_transcript_thread.result.connect(lambda: global_panel_transcription_transcript_thread_result_emmited())
    self.global_panel_transcription_transcript_thread.text.connect(lambda: global_panel_transcription_transcript_thread_text_emmited())

    self.global_panel_transcription_transcript_button = QPushButton('Transcribe')
    self.global_panel_transcription_transcript_button.setProperty('class', 'button')
    self.global_panel_transcription_transcript_button.clicked.connect(lambda: global_panel_transcription_transcript_button_clicked(self))
    self.global_panel_transcription_transcript_groupbox.layout().addWidget(self.global_panel_transcription_transcript_button)

    self.global_panel_transcription_transcript_progress = QProgressBar()
    self.global_panel_transcription_transcript_progress.setVisible(False)
    self.global_panel_transcription_transcript_groupbox.layout().addWidget(self.global_panel_transcription_transcript_progress)

    # self.global_panel_transcription_transcript_button = QPushButton('Auto Subtitle')
    # self.global_panel_transcription_transcript_button.setProperty('class', 'button')
    # self.global_panel_transcription_transcript_button.clicked.connect(lambda: global_panel_transcription_transcript_button_clicked(self))
    # self.global_panel_transcription_transcript_groupbox.layout().addWidget(self.global_panel_transcription_transcript_button)

    # self.global_panel_transcription_transcript_label = QLabel('This will overwrite all of the subtitles on the project.')
    # self.global_panel_transcription_transcript_label.setProperty('class', 'units_label')
    # self.global_panel_transcription_transcript_groupbox.layout().addWidget(self.global_panel_transcription_transcript_label)

    self.global_panel_transcription_transcript_groupbox.layout().addStretch()

    self.global_panel_transcription_transcript_column.addWidget(self.global_panel_transcription_transcript_groupbox)

    self.global_panel_transcription_transcript_column.addStretch()

    self.global_panel_transcription_transcript_widget.layout().addLayout(self.global_panel_transcription_transcript_column)

    self.global_panel_transcription_transcript_preview_column = QVBoxLayout()
    self.global_panel_transcription_transcript_preview_column.setContentsMargins(0, 0, 0, 0)
    self.global_panel_transcription_transcript_preview_column.setSpacing(0)

    self.global_panel_transcription_transcript_preview = global_panel_transcription_transcript_preview()
    self.global_panel_transcription_transcript_preview.setFixedSize(QSize(200, 150))

    self.global_panel_transcription_transcript_preview_column.addWidget(self.global_panel_transcription_transcript_preview)

    self.global_panel_transcription_transcript_preview_column.addSpacing(10)

    self.global_panel_transcription_transcript_slice_label = QLabel('Slicing and timing')
    self.global_panel_transcription_transcript_slice_label.setProperty('class', 'widget_label')
    self.global_panel_transcription_transcript_preview_column.addWidget(self.global_panel_transcription_transcript_slice_label)

    self.global_panel_transcription_transcript_preview_column.addSpacing(2)

    self.global_panel_transcription_transcript_slice_combobox = QComboBox()
    self.global_panel_transcription_transcript_slice_combobox.addItems(['Equal size', 'Phrase size', 'Words'])
    self.global_panel_transcription_transcript_slice_combobox.activated.connect(lambda: global_panel_transcription_transcript_slice_combobox_activated(self))
    self.global_panel_transcription_transcript_preview_column.addWidget(self.global_panel_transcription_transcript_slice_combobox)

    self.global_panel_transcription_transcript_preview_column.addSpacing(10)

    self.global_panel_transcription_transcript_apply_transcript_button = QPushButton('Slice to subtitles')
    self.global_panel_transcription_transcript_apply_transcript_button.setProperty('class', 'button')
    self.global_panel_transcription_transcript_apply_transcript_button.setVisible(False)
    self.global_panel_transcription_transcript_apply_transcript_button.clicked.connect(lambda: global_panel_transcription_transcript_apply_transcript_button_clicked(self))
    self.global_panel_transcription_transcript_preview_column.layout().addWidget(self.global_panel_transcription_transcript_apply_transcript_button)

    self.global_panel_transcription_transcript_preview_column.addStretch()

    self.global_panel_transcription_transcript_widget.layout().addLayout(self.global_panel_transcription_transcript_preview_column)

    self.global_panel_transcription_tabwidget.addTab(self.global_panel_transcription_transcript_widget, 'Transcript')

    self.global_panel_transcription_content.layout().addWidget(self.global_panel_transcription_tabwidget)

    self.global_panel_content_stacked_widgets.addWidget(self.global_panel_transcription_content)


def global_subtitlesvideo_autosync_button_clicked(self):
    """Function to run autosync"""
    run_command = False

    if bool(self.subtitles_list):
        are_you_sure_message = QMessageBox(self)
        are_you_sure_message.setWindowTitle(self.tr('Are you sure?'))
        are_you_sure_message.setText(self.tr('This will overwrite your actual subtitle set. New timings will be applied. Are you sure you want to replace your actual subtitles?'))
        are_you_sure_message.addButton(self.tr('Yes'), QMessageBox.AcceptRole)
        are_you_sure_message.addButton(self.tr('No'), QMessageBox.RejectRole)
        ret = are_you_sure_message.exec_()

        if ret == QMessageBox.AcceptRole:
            run_command = True
    else:
        run_command = True

    if run_command:
        file_io.save_file(os.path.join(path_tmp, 'subtitle_original.srt'), self.subtitles_list, 'SRT')
        sub = os.path.join(path_tmp, 'subtitle_final.srt')

        # unparsed_args = [self.video_metadata['filepath'], "-i", os.path.join(path_tmp, 'subtitle_original.srt'), "-o", sub]

        # parser = ffsubsync.make_parser()
        # args = parser.parse_args(unparsed_args)

        # ffsubsync.run(args)

        if os.path.isfile(sub):
            self.subtitles_list = file_io.process_subtitles_file(sub)[0]
            # update_widgets(self)


def global_panel_transcription_autosubtitle_button_clicked(self):
    """Function to run autosub"""
    run_command = False

    if bool(self.subtitles_list):
        are_you_sure_message = QMessageBox(self)
        are_you_sure_message.setWindowTitle(self.tr('Are you sure?'))
        are_you_sure_message.setText(self.tr('This will overwrite your actual subtitle set. New timings will be applied. Are you sure you want to replace your actual subtitles?'))
        are_you_sure_message.addButton(self.tr('Yes'), QMessageBox.AcceptRole)
        are_you_sure_message.addButton(self.tr('No'), QMessageBox.RejectRole)
        ret = are_you_sure_message.exec_()

        if ret == QMessageBox.AcceptRole:
            run_command = True
    else:
        run_command = True

    if run_command:
        def read_json_subtitles(filename='', transcribed=False):
            final_subtitles = []
            if filename:
                with open(filename, encoding='utf-8') as json_file:
                    data = json.loads(json_file.read())
                    if transcribed:
                        for fragment in data:
                            subtitle = []
                            subtitle.append(float(fragment['start']))
                            subtitle.append(float(fragment['end']) - subtitle[0])
                            subtitle.append(str(fragment['content']))
                            final_subtitles.append(subtitle)
                    else:
                        for fragment in data['fragments']:
                            subtitle = []
                            subtitle.append(float(fragment['begin']))
                            subtitle.append(float(fragment['end']) - float(fragment['begin']))
                            # subtitle.append(codecs.decode(fragment['lines'][0], 'unicode-escape'))
                            # print(fragment['lines'][0])
                            subtitle.append(str(fragment['lines'][0]))
                            final_subtitles.append(subtitle)

            return final_subtitles

        language = LANGUAGE_DICT_LIST[self.global_panel_transcription_language_combobox.currentText()]
        #
        # command = [
        #     FFMPEG_EXECUTABLE,
        #     '-y',
        #     '-f', 'f32le',
        #     '-ac', '1',
        #     '-ar', '48000',
        #     '-i', '-',
        #     '-vn',
        #     os.path.join(path_tmp, 'subtitle.opus')]
        #
        # audio_out = subprocess.Popen(command, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL, stdin=subprocess.PIPE, startupinfo=STARTUPINFO)
        # audio_out.stdin.write(self.video_metadata['audio'].tobytes())
        # audio_out.stdin.close()
        # audio_out.wait()

        # autosub.generate_subtitles(os.path.join(path_tmp, 'subtitle.opus'), output=os.path.join(path_tmp, 'subtitle.json'), src_language=language, dst_language=language, subtitle_file_format='json')

        autosub.generate_subtitles(
            self.video_metadata['filepath'],
            output=os.path.join(path_tmp, 'subtitle.json'),
            src_language=language,
            dst_language=language,
            subtitle_file_format='json',
            ffmpeg_executable=FFMPEG_EXECUTABLE
        )

        if os.path.isfile(os.path.join(path_tmp, 'subtitle.json')):
            final_subtitles = read_json_subtitles(filename=os.path.join(path_tmp, 'subtitle.json'), transcribed=True)
            if final_subtitles:
                self.subtitles_list = final_subtitles

        # update_widgets(self)


def global_panel_transcription_transcript_apply_transcript_button_clicked(self):
    """Function to transcribe using google speech"""
    run_command = False

    if bool(self.subtitles_list):
        are_you_sure_message = QMessageBox(self)
        are_you_sure_message.setWindowTitle(self.tr('Are you sure?'))
        are_you_sure_message.setText(self.tr('This will overwrite your actual subtitle set. New text will be applied. Are you sure you want to replace your actual subtitles?'))
        are_you_sure_message.addButton(self.tr('Yes'), QMessageBox.AcceptRole)
        are_you_sure_message.addButton(self.tr('No'), QMessageBox.RejectRole)
        ret = are_you_sure_message.exec_()

        if ret == QMessageBox.AcceptRole:
            run_command = True
    else:
        run_command = True

    if run_command and self.global_panel_transcription_transcript_preview.text:
        if self.global_panel_transcription_transcript_slice_combobox.currentText() == 'Equal size':
            self.subtitles_list = []
            ns = len(self.global_panel_transcription_transcript_preview.text.split('. '))
            ts = self.video_metadata['duration'] / ns
            c = 0.0
            for sub in self.global_panel_transcription_transcript_preview.text.split('. '):
                self.subtitles_list.append([c, ts, sub + '.'])
                c += ts
        elif self.global_panel_transcription_transcript_slice_combobox.currentText() == 'Phrase size':
            self.subtitles_list = []
            ns = len(self.global_panel_transcription_transcript_preview.text)
            c = 0.0
            for sub in self.global_panel_transcription_transcript_preview.text.split('. '):
                ts = (len(sub + '.') / ns) * self.video_metadata['duration']
                self.subtitles_list.append([c, ts, sub + '.'])
                c += ts
        elif self.global_panel_transcription_transcript_slice_combobox.currentText() == 'Words':
            self.subtitles_list = []
            ns = len(self.global_panel_transcription_transcript_preview.text)
            c = 0.0
            for sub in self.global_panel_transcription_transcript_preview.text.split(' '):
                ts = (len(sub + ' ') / ns) * self.video_metadata['duration']
                self.subtitles_list.append([c, ts, sub])
                c += ts


def global_panel_transcription_transcript_button_clicked(self):
    self.global_panel_transcription_transcript_thread.metadata = self.video_metadata
    self.global_panel_transcription_transcript_thread.selected_language = self.global_panel_transcription_language_combobox.currentText()
    self.global_panel_transcription_transcript_thread.start()


def global_panel_transcription_transcript_slice_combobox_activated(self):
    if self.global_panel_transcription_transcript_slice_combobox.currentText() == 'Equal size':
        self.global_panel_transcription_transcript_preview.update_type('equal')
    elif self.global_panel_transcription_transcript_slice_combobox.currentText() == 'Phrase size':
        self.global_panel_transcription_transcript_preview.update_type('phrases')
    elif self.global_panel_transcription_transcript_slice_combobox.currentText() == 'Words':
        self.global_panel_transcription_transcript_preview.update_type('words')



"""Software update module

"""

import os
import requests

try:
    import _winreg as winreg
except ImportError:
    import winreg

from PySide6.QtCore import QThread, Signal

from subtitld.modules.paths import VERSION_NUMBER, PATH_SUBTITLD_DATA_UPDATE



class ThreadCheckVersion(QThread):
    """Class for QThread for checking version"""
    command = Signal(dict)
    url = 'https://api2.jonata.org/subtitld/windows_version'

    def run(self):
        """Function for running QThread"""
        result = {}
        result = requests.get(self.url).json()
        self.command.emit(result)


class ThreadDownloadInstaller(QThread):
    """Class for QThread for download installer"""
    command = Signal(str)
    url = 'https://api2.jonata.org/subtitld/get_windows_installer'
    downloadfolder = os.path.join(PATH_SUBTITLD_DATA_UPDATE, 'subtitld_update.exe')

    def run(self):
        """Function for running QThread"""
        if self.url and os.path.isdir(os.path.dirname(self.downloadfolder)):
            result = ''
            with requests.get(self.url, stream=True) as req:
                req.raise_for_status()
                with open(self.downloadfolder, 'wb') as fileupd:
                    for chunk in fileupd.iter_content(chunk_size=8192):
                        fileupd.write(chunk)
                if os.path.isfile(self.downloadfolder):
                    result = self.downloadfolder
            self.command.emit(result)


def set_run_key(key, value):
    """Function to call registry and schedule or remove installer on startup"""
    reg_key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, r'Software\Microsoft\Windows\CurrentVersion\Run', 0, winreg.KEY_SET_VALUE)

    with reg_key:
        if value is None:
            winreg.DeleteValue(reg_key, key)
        else:
            if '%' in value:
                try:
                    var_type = winreg.REG_EXPAND_SZ
                except Exception:
                    pass
            else:
                var_type = winreg.REG_SZ
            winreg.SetValueEx(reg_key, key, 0, var_type, value)


def load(self):
    """Function to load update"""
    if os.path.isfile(os.path.join(PATH_SUBTITLD_DATA_UPDATE, 'subtitld_update.exe')):
        os.remove(os.path.join(PATH_SUBTITLD_DATA_UPDATE, 'subtitld_update.exe'))
    set_run_key('subtitld_update', None)

    def thread_download_installer_ended(command):
        set_run_key('subtitld_update', command + ' /S')

    self.thread_download_installer = ThreadDownloadInstaller(self)
    self.thread_download_installer.command.connect(thread_download_installer_ended)
    self.thread_download_installer.start()

    def thread_check_version_ended(command):
        if not command.get('stable', VERSION_NUMBER) == VERSION_NUMBER:
            self.thread_download_installer.start()

    self.thread_check_version = ThreadCheckVersion(self)
    self.thread_check_version.command.connect(thread_check_version_ended)
    self.thread_check_version.start()
